#include "arc_source.h"

#include <assert.h>
#include "math/geo.h"

using namespace arc;

static const QString		ARC_NAME("source");
static const QString		TYPE_NAME("type");

/* SOURCE
 ******************************************************************/
Source::Source(const int source)
	: mSource(source)
{
}

Source::Source(const Source& o)
	: Arc(o)
	, mSource(o.mSource)
{
}

Source* Source::copy() const
{
	return new Source(*this);
}

void Source::getParams(param::List&, param::GetArgs&) const
{
}

param::Parent* Source::findParent(param::UrlIterator&)
{
	return NULL;
}

void Source::setParam(const param::UrlIterator&, param::SetArgs&)
{
}

void Source::work(WorkParams& p, float* row)
{
	const float*		src = NULL;
	if (mSource == DISTANCE_SRC && p.hasFlag(HAS_DISTANCE_SRC)) src = p.mDistanceRow;
	else if (mSource == NORMAL_SRC && p.hasFlag(HAS_NORMAL_SRC)) src = p.mNormalRow;

	if (src != NULL) memcpy(row, src, sizeof(float)*p.mRowSize);
}

/* IO
 ******************************************************************/
void Source::writeJsonObject(json::Writer& w) const
{
	writeJsonHeader(w, ARC_NAME);

	w.key(TYPE_NAME);
	w.intValue(mSource);

	writeJsonFooter(w);
}

void Source::jsonFloat(json::Stack& s, const QString& k, const float v)
{
	if (k == TYPE_NAME) mSource = geo::iroundf(v);
	else Arc::jsonFloat(s, k, v);
}

#ifdef QT_DEBUG
void Source::printTree(const int tabs) const
{
	QDebug		d = qDebug();
	for (int k=0; k<tabs; k++) (d.nospace()) << "\t";

	(d.nospace()) << ARC_NAME;
	(d.nospace()) << " id=";
	(d.nospace()) << mId;
	(d.nospace()) << " source=";
	(d.nospace()) << mSource;
}
#endif
