#ifndef ARC_SOURCE_H
#define ARC_SOURCE_H

#include "engine/arc.h"

namespace arc {

/* SOURCE
 * A special arc for replacing the work row with a specific
 * type of data from the params (such as distance or normal).
 * This solves a basic problem with arcs:  How to write a general
 * arc that works on normal input (such as an input that's already
 * the distance values) and also works on multiple types of sources.
 * The way the source works, if it's desired value is in the work
 * params, it replaces, otherwise it leaves the row alone, so the
 * client effectively controls everything.
 ******************************************************************/
class Source : public Arc
{
public:
	explicit Source(const int source = DISTANCE_SRC);
	Source(const Source&);

	virtual Source*			copy() const;

	virtual void			getParams(param::List&, param::GetArgs&) const;
	virtual param::Parent*	findParent(param::UrlIterator&);
	virtual void			setParam(const param::UrlIterator&, param::SetArgs&);

	virtual void			work(WorkParams&, float* row);

	// IO
	virtual void			writeJsonObject(json::Writer&) const;
	virtual void			jsonFloat(json::Stack&, const QString&, const float);

#ifdef QT_DEBUG
	virtual void			printTree(const int tabs) const;
#endif

private:
	int						mSource;
};

} // arc

#endif // ARC_SOURCE_H
