#ifndef BINARY_H
#define BINARY_H

#include "engine/arc.h"
#include "engine/archead.h"

namespace arc {

/* BINARY
 * A binary operation (mult, add, etc)
 ******************************************************************/
class Binary : public Arc
{
public:
	static const int		OP_MULT = 0;

public:
	Binary(const Binary&);
	explicit Binary(const int op = OP_MULT);
	Binary(const int op, Arc* left, Arc* right);

	virtual Binary*			copy() const;

	virtual void			getParams(param::List&, param::GetArgs&) const;
	virtual param::Parent*	findParent(param::UrlIterator&);
	virtual void			setParam(const param::UrlIterator&, param::SetArgs&);

	virtual void			work(WorkParams&, float* row);

	virtual int				forEach(arc::Visitor&);

	// IO
	virtual void			writeJsonObject(json::Writer&) const;
	virtual void			startObject(json::Stack&, const QString&);

#ifdef QT_DEBUG
	virtual void			printTree(const int tabs) const;
#endif

private:
	int						mOp;
	ArcHead					mLeft, mRight;
};

} // arc

#endif // BINARY_H
