#ifndef RECYCLEARRAY_H
#define RECYCLEARRAY_H

#include <string.h>

/* RECYCLE-ARRAY
 * A simple array that doesn't free memory when it shrinks, instead
 * saving the entries to be reused later.
 * I'd really like to just use STL array or something in the platform,
 * but as far as I know, there are never any guarantees as to how
 * other collections will be implemented, and I specifically want
 * something that avoids memory allocation and deallocation.
 ******************************************************************/
template <class T>
class RecycleArray
{
public:
	RecycleArray();
	virtual ~RecycleArray();

	// Answer data that's the requested size, allocating if
	// necessary.
	T*				get(const int size);

private:
	T*				mD;
	int				mAlloc;
};

/* RECYCLE-ARRAY IMPLEMENTATION
 ******************************************************************/
template <class T>
RecycleArray<T>::RecycleArray()
	: mD(NULL)
	, mAlloc(0)
{
}

template <class T>
RecycleArray<T>::~RecycleArray()
{
	realloc(mD, 0);
}


template <class T>
T* RecycleArray<T>::get(const int size)
{
	if (size <= 0) return NULL;
	if (size > mAlloc) {
		T*			newD = (T*)realloc(mD, sizeof(T)*size);
		if (newD == NULL) return NULL;
		mD = newD;
		mAlloc = size;
	}
	return mD;
}

#endif // RECYCLEARRAY_H
