#ifndef ARC_H
#define ARC_H

#include "arcdefs.h"
#include "engine/engineparam.h"
#include "io/json.h"

// Forward declarations
namespace arc {
	class FillId;
}

/* ARC
 * A simple f(x) value mapper.
 ******************************************************************/
class Arc : public param::Parent
		  , public json::Reader
{
public:
	virtual ~Arc();

	virtual Arc*			copy() const = 0;

	// Follow the URL to find the parent at the given param.
	virtual param::Parent*	findParent(param::UrlIterator&) = 0;

	virtual int				forEach(arc::Visitor&);

	// The size of row is defined in RunParams
	virtual void			work(arc::WorkParams&, float* row) = 0;

	// IO
	virtual void			writeJsonObject(json::Writer&) const = 0;
	// Default behaviour for reading in the ID.
	virtual void			jsonFloat(json::Stack&, const QString&, const float);

#ifdef QT_DEBUG
	virtual void			printTree(const int tabs) const = 0;
#endif

protected:
    Arc();
	Arc(const Arc&);
	friend class arc::FillId;

	// A convenience since all clients should have the same header / footer
	void					writeJsonHeader(json::Writer&, const QString& key) const;
	void					writeJsonFooter(json::Writer&) const;

private:
	void					operator=(const Arc&);
};

#endif // ARC_H
