#include "archead.h"

#include <stdlib.h>

#include <QDebug>

#include "engine/arc.h"
#include "engine/arcdefs.h"
#include "engine/fxdefs.h"

ArcHead::ArcHead()
	: mArc(NULL)
{
}

ArcHead::ArcHead(const ArcHead& o)
	: param::Parent(o)
	, json::Reader(o)
	, arc::Reader::Target(o)
	, mArc(NULL)
	, mKey(o.mKey)
{
	if (o.mArc != NULL) mArc = o.mArc->copy();
	mLinks.setTo(o.mLinks);
}

ArcHead::~ArcHead()
{
	delete mArc;
}

Arc* ArcHead::getArc()
{
	return mArc;
}

void ArcHead::setId(const int id)
{
	mId = id;
}

QString ArcHead::getKey() const
{
	return mKey;
}

void ArcHead::setKey(const QString& key)
{
	mKey = key;
}

void ArcHead::setArc(Arc* a)
{
	delete mArc;
	mArc = a;
}

void ArcHead::setTo(const ArcHead& o)
{
	setContentTo(o);
	// I guess I should do the ID for this, too, but
	// maybe I should just get rid of this function.
}

void ArcHead::setContentTo(const ArcHead& o)
{
	setArc(NULL);
	if (o.mArc != NULL) mArc = o.mArc->copy();
	mKey = o.mKey;
	mLinks.setTo(o.mLinks);
}

param::Parent* ArcHead::findParent(param::UrlIterator& it)
{
	// I think this will get a little more complicated once
	// heads can store param lists, but for now, it's just
	// a passthrough.
	if (mArc == NULL) return NULL;
	if (!it.matches(mArc->getId())) return NULL;
	++it;
	return mArc->findParent(it);
}

void ArcHead::getParams(param::List& list, param::GetArgs& get) const
{
	if (mArc == NULL) return;

	param::List			l;
	mArc->getParams(l, get);
	l.prepend(mArc->getId());
	l.prepend(mId);
	list.takeAll(l);
}

void ArcHead::setParam(const param::UrlIterator&, param::SetArgs&)
{
}

void ArcHead::addTypeParam(	param::List& list, param::GetArgs& get,
							const int bit, const int flags) const
{
	const float				exp = param::valueFromFlag(flags, bit);
	param::List				group;
	getParams(group, get);
	list.addTypeParam(fx::ARC_SZ, mId, param::ARC_BROWSER_TYPE, get, exp, bit, &mKey, &group);
}

void ArcHead::addLink(const arc::ModLink& link)
{
	mLinks.mLink.append(link);
}

int ArcHead::forEach(arc::Visitor& v)
{
	if (mArc == NULL) return arc::CONTINUE;
	return mArc->forEach(v);
}

#ifdef QT_DEBUG
void ArcHead::printTree(const int tabs) const
{
	if (mArc == NULL) return;
	mArc->printTree(tabs);
}
#endif

/* IO
 ******************************************************************/
void ArcHead::writeJsonObject(json::Writer& w, const QString& key) const
{
	w.key(key);
	w.beginObject();
	writeJson(w);
	w.endObject();
}

void ArcHead::writeJson(json::Writer& w) const
{
	if (mKey.length() > 0) {
		w.key(fx::KEY_SZ);
		w.stringValue(mKey);
	}

	if (mArc != NULL) {
		w.key(fx::DATA_SZ);
		mArc->writeJsonObject(w);
	}
}

void ArcHead::startObject(json::Stack& s, const QString& k)
{
	if (k == fx::DATA_SZ) s.push(new arc::Reader(*this));
}

void ArcHead::jsonString(json::Stack&, const QString& k, const QString& v)
{
	if (k == fx::KEY_SZ) mKey = v;
}

void ArcHead::parseArc(Arc* a)
{
	delete mArc;
	mArc = a;
}
