#ifndef ARCHEAD_H
#define ARCHEAD_H

#include "engine/arcmod.h"
#include "engine/engine_io.h"
#include "engine/parent.h"
#include "io/json.h"

class Arc;

namespace arc {
	class Visitor;
	class WorkParams;
}

/* ARC-HEAD
 * Store a single arc.  Convenience so clients don't
 * have to memory manage.
 ******************************************************************/
class ArcHead : public param::Parent
			  , public json::Reader
			  , public arc::Reader::Target
{
public:
    ArcHead();
	ArcHead(const ArcHead&);
	~ArcHead();

	bool				isEmpty() const;

	// The ID for a head is generally set by the object that contains
	// it, as a way to uniquely identify it from other parameters.
	void				setId(const int);

	// ArcHeads are identified by a unique structured string.
	// The format is "creator:name", i.e. the default Angry Red
	// Planet S*N arc is "arp:S*N".
	QString				getKey() const;
	void				setKey(const QString&);

	Arc*				getArc();
	void				setArc(Arc*);
	void				setTo(const ArcHead&);
	// Used by clients to set the contents, leaving the ID alone.
	void				setContentTo(const ArcHead&);

	param::Parent*		findParent(param::UrlIterator&);

	void				getParams(param::List&, param::GetArgs&) const;
	virtual void		setParam(const param::UrlIterator&, param::SetArgs&);
	// Convenience to add myself as a type param, with the given state.
	void				addTypeParam(	param::List& list, param::GetArgs& get,
										const int bit, const int flags) const;

	void				addLink(const arc::ModLink&);

	int					forEach(arc::Visitor&);

	// IO
	void				writeJsonObject(json::Writer&, const QString& key) const;
	virtual void		startObject(json::Stack&, const QString&);
	virtual void		jsonString(json::Stack&, const QString&, const QString&);
	virtual void		parseArc(Arc*);

#ifdef QT_DEBUG
	void				printTree(const int tabs) const;
#endif

protected:
	friend class arc::WorkParams;

	arc::ModTable		mLinks;

private:
	void				operator=(const ArcHead&);

	Arc*				mArc;
	QString				mKey;

	void				writeJson(json::Writer&) const;
};

/* inline
 ******************************************************************/
inline bool ArcHead::isEmpty() const
{
	return mArc == NULL;
}

#endif // ARCHEAD_H
