#ifndef ARCMOD_H
#define ARCMOD_H

#include <QVector>

#include "math/primitives/range.h"
#include "compositing/mask.h"

class Arc;

namespace arc {

/* ARC::MOD-SRC
 * A single modulation source and its current values.
 ******************************************************************/
class ModSrc
{
public:
	static const int		STATE_ON = (1<<0);

public:
	int						mState;
//	ArcHead					mArc;
	Mask					mRow;

public:
	ModSrc();

	void					setIsOn(const bool);

	// Answer my row of data for the requested size, but
	// only if I'm currently on.
	float*					row(const int size);
};

/* ARC::MOD-LINK
 * A single src to dst connection
 ******************************************************************/
class ModLink
{
public:
	static const int		RANGE_ON_F = (1<<0);

public:
	int						mFlags;
	int						mSrc;
	int						mDstArc, mDstParam;
	Range					mR;

public:
	ModLink();
	ModLink(const ModLink&);
	ModLink(const int src, const int dstArc, const int dstParam,
			const float min, const float max);
	// Use this variant when you don't want any range translation,
	// but instead just a direct assignment of the source value
	ModLink(const int src, const int dstArc, const int dstParam);

	void					operator=(const ModLink&);
};

/* ARC::MOD-TABLE
 * A collection of links.
 ******************************************************************/
class ModTable
{
public:
	QVector<ModLink>		mLink;

	void					setTo(const ModTable&);

public:
	ModTable();
};

} // namespace arc

#endif // ARCMOD_H
