#include "celldistance.h"

#include <math.h>
#include "math/cppolygon.h"

#include <QTime>

#ifdef GENERATE_NORMALS
#include "fx/bevel_support.h"
#endif

static float poly_dist(const Point3& pt, const QPolygonF& poly, Segment3& out);

/* CELL-DISTANCE
 ******************************************************************/
CellDistance::CellDistance()
{
}

void CellDistance::cache(Cell* cell, const QSize& size, fx::WorkArgs& wa)
{
	// If the distance is already cached, I can bail.
	// Distance caching is a special case, it's locked to the shape
	if (wa.shapeStamp() == wa.distanceStamp()) return;
	const int			w = size.width(), h = size.height();
	const QPolygonF&	shape = wa.viewShape();
	if (shape.size() < 2) return;

	CpPolygon	cp;
	cp.setTo(shape);
	Segment3		triAb;
	QTime			time;
	time.start();

	int					pix = 0;
	Point3				pt(0, 0, 0);
	QPointF				qpt;
	float				minD = 1000000000,
						maxD = -1000000000;
#ifdef GENERATE_NORMALS
	bevel::Normal		nrm;
	Point3				nrmPt;
#endif
	for (int y=0; y<h; y++) {
		pt.y = y;
		qpt.setY(pt.y);
		for (int x=0; x<w; x++) {
			pt.x = x;
			qpt.setX(pt.x);
			pt.z = poly_dist(pt, shape, triAb);
#ifdef GENERATE_NORMALS
		nrm.setOn(triAb.mPt[0], pt, triAb.mPt[1], nrmPt);
		cell[pix].mNormalX = nrmPt.x;
		cell[pix].mNormalY = nrmPt.y;
		cell[pix].mNormalZ = nrmPt.z;
#endif

			float		d = cp.distanceTo(x, y);
	/*
	if (y == 100) {
		const float		d2 = cp.distanceTo(x, y);
		QDebug			db = qDebug();
		db << "(" << x << "," << y << ") old=" << d << " new=" << d2;
	}
	*/
			if (!shape.containsPoint(qpt, Qt::OddEvenFill)) d = -d;
			if (d < minD) minD = d;
			if (d > maxD) maxD = d;
			cell[pix].mD = d;
			pix++;
		}
	}
//	qDebug() << "elapsed=" << time.elapsed();
	wa.distanceCached(minD, maxD);
}

/* misc
 ******************************************************************/
static float poly_dist(const Point3& pt, const QPolygonF& poly, Segment3& out)
{
	// OMG!  So slow!  Probably should store line segments in an
	// easy-to-search format
	// One small optimization is eliminating the sqrt() from each
	// distance test, although on my machine, this barely shaves
	// off any time, so clearly this architecture has a sqrt
	// optimizer, but presumably other platforms won't.
	const int			size = poly.size();
	Segment3			seg;
	seg.set(poly[size-1], poly[0]);
	float				min = seg.distanceToSquared(pt);
	out = seg;
	for (int k=1; k<size; k++) {
		seg.set(poly[k-1], poly[k]);
		const float		d = seg.distanceToSquared(pt);
		if (d < min) {
			min = d;
			out = seg;
		}
	}
	return sqrt(min);
}

#undef GENERATE_NORMALS
