#ifndef COLORF_H
#define COLORF_H

#include <QString>

class QColor;

/* COLOR-F
 * Store a color in our standard format, floating point values.
 * Provides conveniences for altering to alpha premultiplied.
 * All of the _Pm members assume values are already premultipled.
 ******************************************************************/
struct color_f
{
	float			r, g, b, a;

	color_f();
	color_f(const color_f&);

	color_f&		operator=(const color_f&);

	// Set based on non-premultiplied 8-bit values
	void			setRgba(const int r, const int g, const int b, const int a);
	// Set to the incoming color, assumed to be unmatted, and premultiply
	void			setAndPremultiply(const color_f&);

	// Answer myself as a QColor
	void			getColor(QColor&) const;

	// Convert from unmatted to matted.
	void			premultiply(const float a);
	// Convert from matted back to unmatted.
	void			unPremultiply_Pm();

	// Set the alpha value, assuming I am premultiplied and that needs to be maintained.
	void			setA_Pm(const float a);

	// Converting
	QString			asHex() const;				// RGBA, i.e. "00ff0088" =  green, half opaque
	bool			fromHex(const QString&);
};

QDebug& operator<<(QDebug&, const color_f&);

/* COMPOSITING
 * The compositing commands taking a source colour and destination
 * image data.  The dst is stored as floats in RGBA, but the pix
 * argument treats it like an array of single values, that is, it
 * will get multiplied by 4.  Implementations are the standard
 * Porter-Duff compositing modes.
 ******************************************************************/
void srcOver(const color_f& src, float* dst, int pix);


#endif // COLORF_H
