#ifndef ENGINE_IO_H
#define ENGINE_IO_H

#include "io/json_reader.h"

#include <QString>

/* ENGINE-IO
 * Lose collection of classes involved with parsing.
 ******************************************************************/

// PARSING CONSTANTS
extern const QString&	CLR_SZ;

class Arc;
class ColorSrc;

namespace arc {

/* ARC::READER
 * A generic class clients can instantiate to read in arcs.
 ******************************************************************/
class Reader : public json::Reader
{
// Clients need to supply a target for all my arc reading efforts.
public:
	class Target {
	public:
		virtual ~Target() { }

		virtual void	parseArc(Arc*) = 0;

	protected:
		Target() { }
		Target(const Target&) { }
	};

public:
	explicit Reader(Target&);

	virtual void		jsonString(json::Stack&, const QString&, const QString&);
	virtual void		jsonPop();

private:
	Target&				mT;
};

} // namespace arc

/* COLOR-SRC-TARGET
 * Interface for classes that will receive results of parsing colors.
 ******************************************************************/
class ColorSrcTarget {
public:
	virtual ~ColorSrcTarget() { }

	virtual void	parseClr(ColorSrc*) = 0;

protected:
	ColorSrcTarget() { }
	ColorSrcTarget(const ColorSrcTarget&) { }
};

#endif // ENGINE_IO_H
