#ifndef ENGINEPARAM_H
#define ENGINEPARAM_H

#include <QString>
#include "param/param.h"
#include "engine/archead.h"
#include "engine/colorsrchead.h"

/* A few classes that straddle the boundary between the param
 * and engine modules, adding some engine-specific functionality
 * to the param framework.  From the apps perspective, it should
 * look like these things are actually in the param module, hence
 * that's the namespace used.
 ******************************************************************/

namespace param {

static const int		ARC_BROWSER_TYPE = Param::CLIENT_TYPE;
static const int		COLOR_BROWSER_TYPE = Param::CLIENT_TYPE+1;
static const int		SHAPE_BROWSER_TYPE = Param::CLIENT_TYPE+2;
static const int		RAW_FLOAT_TYPE = Param::CLIENT_TYPE+3;
	// for floats that don't use normalized 0-1 values

static const int		ARC_FEATURE = (1<<0);
static const int		COLOR_FEATURE = (1<<1);

// FX might have a FRAME_SZ group containing the params.  Clients
// can treat these as raw coordinates to manipulate.
extern const QString&	FRAME_SZ;
extern const QString&	LEFT_SZ;
extern const QString&	TOP_SZ;
extern const QString&	RIGHT_SZ;
extern const QString&	BOTTOM_SZ;

// Conversion between bit flag and param value.
const float				valueFromFlag(const int v, const int flagBit);
const int				flagFromValue(const float v, const int flagBit);
const bool				isExpanded(const float v);

/* PARAM::SET-ARGS.  Arguments for setParam(), keeps everything
 * bundled in case the API changes over time.
 ******************************************************************/
struct SetArgs {
	// Input
	float				mV;
	QString				mStr;
	ArcHead				mArc;
	ColorSrcHead		mClr;
	// Output
	bool				mChanged;

	SetArgs();
	SetArgs(const SetArgs&);
	void				operator=(const SetArgs&);

	void				clear();

	void				readFrom(const Param&);
	void				writeTo(Param&) const;
};

} // namespace param

#endif // ENGINEPARAM_H
