#include "fxhead.h"

#include "engine/fx.h"

/* FX-HEAD
 ******************************************************************/
FxHead::FxHead()
	: mFx(NULL)
{
}

FxHead::~FxHead()
{
	setFx(NULL);
}

void FxHead::setFx(Fx* fx)
{
	// This should never be true, but in the offhand case
	// that something weird happens, I'll protect against it.
	if (fx == mFx) return;

	delete mFx;
	mFx = fx;
}

void FxHead::getParams(param::List& list, param::GetArgs& get) const
{
	if (mFx != NULL) {
		mFx->getParams(list, get);
		list.prepend(mFx->getId());
	}
}

param::Parent* FxHead::findParent(const param::Url& url) const
{
	if (mFx == NULL) return NULL;
	param::UrlIterator		it(url);
	if (!it.hasValue()) return NULL;
	if (!it.matches(mFx->getId())) return NULL;

	++it;
	return mFx->findParent(it);
}

void FxHead::swap(FxHead& o)
{
	Fx*		fx = mFx;	mFx = o.mFx;	o.mFx = fx;
}

void FxHead::forEach(fx::Visitor& v, fx::VisitParams& vp)
{
	if (mFx != NULL) mFx->forEach(v, vp);
}

int FxHead::edit(fx::EditOp& op)
{
	if (mFx != NULL) return mFx->edit(op);
	return fx::CONTINUE;
}
