#ifndef BEVEL_H
#define BEVEL_H

#include "engine/fx.h"
#include "model/context.h"

#include "fx/bevel_cellnormal.h"
#include "fx/light.h"

namespace fx {

/* FX::BEVEL
 * Peform a beveling operation.  Requires
 * an input polygon.
 ******************************************************************/
class Bevel : public Fx
{
public:
	Bevel();
	virtual ~Bevel();

	virtual Bevel*			copy() const;

	virtual void			init(fx::Setup&);

	virtual QString			getLabel() const;

	virtual void			getParams(param::List&, param::GetArgs&) const;
	virtual param::Parent*	findParent(param::UrlIterator&);
	virtual void			setParam(const param::UrlIterator&, param::SetArgs&);

	virtual void			work(fx::WorkArgs&, fx::WorkOutput&);

	// IO
	virtual void			writeJson(json::Writer&) const;
	virtual void			startObject(json::Stack&, const QString&);
	virtual void			jsonFloat(json::Stack&, const QString&, const float);

private:
	Bevel(const Bevel&);

	float					mDepth;
	float					mLightAngle;
	float					mWire;

	bevel::CellNormal		mCn;

	// Offload the actual rendering
	Light					mLight;
};

/* FX::BEVEL-FACTORY
 ******************************************************************/
class BevelFactory : public FxFactory
{
public:
	BevelFactory();

	virtual QString			label() const;
	virtual QString			key() const;
	virtual float			match(const QString& key) const;
	virtual Fx*				create() const;
};

} // fx

#endif // BEVEL_H
