#ifndef BEVEL_CELLNORMAL_H
#define BEVEL_CELLNORMAL_H

#include "collections/recyclearray.h"
#include "engine/fxdefs.h"
#include "math/tests/segplane.h"
#include "bevel_edgebuilder.h"
#include "bevel_source.h"
#include "bevel_surface.h"

namespace bevel {

// Equation for finding a light value given a normal and global
inline float lightFrom(const Point3& normal, const Point3& light, const float length)
{
	return (normal.x*light.x + normal.y*light.y + normal.z*light.z) / (normal.length() * length);
}

/* BEVEL::CELL-NORMAL.  A generator for the normal info in a Cell.
 ******************************************************************/
class CellNormal : public fx::CellGenerator
{
public:
	CellNormal();

	void					setDepth(const float depth);

	virtual void			cache(Cell*, const QSize&, fx::WorkArgs&);

	// Get the lowest and highest light value based on the geometry
	// and the light source.
	void					getLightRange(const Point3& light, float& min, float& max) const;

private:
	qint64					mShapeStamp;
	bool					mNeedsCache;
	int						mCreated;

	// Retain the extruded sources, so that I have access to the
	// normal info downstream.
	SourceList				mSources;

	Polygon					mShape;
	float					mDepth;
	EdgeBuilder::State		mEbState;
	SurfaceNormal			mSrfNrm;

	math::SegPlane			mSP;

	// Cache the surface found for each pixel, which can greatly decrease
	// lookup times.
	RecycleArray<const Surface*> mRow;

	void					init(Cell*, const QSize&) const;

	// LOW QUALITY
	bool					lowQuality(Cell*, const QSize& size);
	void					setPixelNormals(Cell*, const QSize&, const SourceList&);

	// HIGH QUALITY
	bool					highQuality(Cell*, const QSize& size);
	void					buildEdges(SurfaceList& allA);
	void					setPixelNormals(Cell*, const QSize&, const SurfaceList&);

	// LOW QUALITY
	bool					testQuality(Cell*, const QSize& size);
//	void					setPixelNormals(Cell*, const QSize&, const SourceList&);
};

} // namespace bevel

#endif // BEVEL_CELLNORMAL_H
