#include "bevel_clip.h"

#include "math.h"

#include "math/geo.h"
#include "bevel_source.h"
#include "bevel_surface.h"

using namespace bevel;

/* BEVEL::CLIP
 ******************************************************************/
Clip::Clip()
	: mDepth(0)
{
}

void Clip::setExtension(const Point3& extensionVec, const float depth)
{
	mExt.set(extensionVec);
	mDepth = depth;
}

void Clip::on(	const Polygon& shape, const Source& a,
				const SourceList& allB, SurfaceList& output)
{
	// Clip the source against allSrc, placing the results
	// into the output
	a.mPoly.getFrame(mFrameA);
	mMask.start(mDepth);

	SourceListIterator			itB(allB);
int		cc = -1;
	while (itB.hasValue()) {
cc++;
		const Source*			b = *itB;
		++itB;
		if (b == NULL || &a == b) continue;
//if (!(cc >= 2 && cc <= 4)) continue;
//if (!(cc == 0)) continue;
//if (mPrint) qDebug() << "attempt to add clipper " << cc << " from " << b->mPoly;
		addClipper(a, b->mPoly);
	}
	// Clip the dst.
mMask.mPrint = mPrint;
	mMask.clip(shape, a, output);
}

void Clip::addClipper(const Source& a, const Polygon& b)
{
	b.getFrame(mFrameB);
	// Rely on the fact that we have a fixed overhead viewing
	// angle, so if frames don't overlap in XY space, they
	// won't intersect at all.
	if (!mFrameA.intersects(mFrameB)) return;

	// Get a poly describing the intersection of the source
	// plane and the cmp tri.  Add the poly to the clip.
	// Note that it's possible for the dst poly to be collapsed
	// down to as small as a single point based on how things
	// intersect.  Which, obviously, is not right.
	mPolyB.clear();
	if (getClipper(a.mPlane, b, mPolyB) && mPolyB.getSize() > 2) {
//	for (int k=0; k<mPolyB.mSize; k++) mDisplayPts1.add(new Point3(mPolyB.mPt[k]));
		// Remove duplicate points.
		mPolyB.compress(0.01);
		if (mPolyB.getSize() > 2) {
			mOrder.makeCcw(mPolyB.getPoints(), mPolyB.getSize());
			mMask.addClipper(mPolyB);
if (mPrint) qDebug() << "clipper is " << mPolyB;
		}
	}
}

bool Clip::getClipper(const Plane& srcP, const Polygon& org, Polygon& clp)
{
	const qint32			osize = org.getSize();
	const Point3*			opts = org.getPoints();
	if (!mPolyPrj.setSize(osize)) return false;
	Point3*					prjpts = mPolyPrj.getPoints();

	// Find where the plane intersects each volume vertex of the tri
	for (int k=0; k<osize; k++) {
		mSegA.set(opts[k], opts[k]);
		mSegA.mPt[1] += mExt;
		const int		ans = mLP.on(mSegA, srcP, &(prjpts[k]));
		if (ans <= math::LinePlane::ERROR) {
//			System.out.println("Clipper.getDstClip error finding intersection");
			return false;
		// The segment is on the plane.
		} else if (ans < math::LinePlane::INTERSECT) {
//			System.out.println("Clipper.getDstPoly line coincident, so which point?");
//			System.out.println("\t" + mSegA);
			return false;
		}
	}

	int*					code = mI.get(osize+8);
	if (code == NULL) return false;

//qDebug() << "projection=" << mPolyPrj;

	// I now have a polygon lying on the src plane.  Let's see about masking.
	int						case1 = 0, case2 = 0;
	for (int k=0; k<osize; k++) {
		const Point3		&spt = prjpts[k], &opt = opts[k];
		const double		d = geo::abs(opt.z - spt.z);
		// Original point is on or below the source plane
		if (d < 0.001 || opt.z < spt.z) {
			code[k] = 0;
			case1++;
		// Original point is above the source plane
		} else {
			code[k] = 1;
			case2++;
		}
//qDebug() << "\t" << k << " code=" << code[k] << " d=" << d << " sub=" << (opt.z - spt.z) << " opt=" << opt.z << " spt=" << spt.z;
	}
//qDebug() << "case1=" << case1 << " case2=" << case2 << " size=" << osize;

	// Handle the original poly completely masking (below) the src
	if (case1 >= osize) {
		clp.set(mPolyPrj);
		return true;
	// Handle the original poly completely above the src
	} else if (case2 >= osize) {
		return false;
	}

	// Any pts below are added straight in, any above are found
	// where the intersection is with the one below.
	const Point3*			oprv = &(opts[osize-1]);
	int						iprv = code[osize-1];
	for (int k=0; k<osize; k++) {
		const Point3		&onxt = opts[k], &snxt = prjpts[k];
		const int			inxt = code[k];
//qDebug() << "k=" << k << " code=" << inxt << " prv=" << iprv;
		// This point is below or equal, so add it directly
		if (inxt <= 0) {
			// If the previous was above, nothing was added for it, so
			// find the intersection.
			if (iprv > 0) {
				mSegA.set(*oprv, onxt);
				if (mSP.on(mSegA.mPt[0], mSegA.mPt[1], srcP, mPtA) >= math::SegPlane::INTERSECT) {
					clp.add(mPtA);
//qDebug() << "\tadd 1 " << mPtA;
				}
			}
			// Now add in this point
			clp.add(snxt);
//qDebug() << "\tadd 2 " << snxt;
		// This point is above.  If the previous is below or equal,
		// find the segment intersection with the src plane.
		} else if (iprv <= 0) {
			mSegA.set(*oprv, onxt);
//qDebug() << "\tseg=" << mSegA.mPt[0] << ", " << mSegA.mPt[1];
			if (mSP.on(mSegA.mPt[0], mSegA.mPt[1], srcP, mPtA) >= math::SegPlane::INTERSECT) {
				clp.add(mPtA);
//qDebug() << "\tadd 3 " << mPtA;
			}
		}
		oprv = &onxt;
		iprv = inxt;
	}
	return true;
}
