#ifndef BEVEL_CLIP_H
#define BEVEL_CLIP_H

#include "math/primitives/polygon.h"
#include "math/tests/lineplane.h"
#include "math/tests/segplane.h"
#include "math/mathutils.h"
#include "fx/bevel_mask.h"

namespace bevel {
class Source;
class SourceList;
class SurfaceList;

/* BEVEL::CLIP.  Clip all overlapping polygons.
 ******************************************************************/
class Clip
{
public:
	bool				mPrint;

public:
	Clip();

	// The extensionVec is a vector describing how to extend the
	// target triangle into a 3d volume.  For my purposes, it will
	// always just be a point of x=0,y=0,z=something large enough
	// to cover the bevel depth.
	void				setExtension(const Point3& extensionVec, const float depth);

	// Shape is the original polygon, used to eliminate external results.
	void				on(	const Polygon& shape, const Source& src,
							const SourceList& allSrc, SurfaceList& output);

private:
	// Clients need to set this vec to something that will generate
	// the proper mesh object based on the cmp tris.  That is, turn
	// a flat tri into a 3d volume that can be clipped against.
	// In practice, it will always be an x,y of 0 and a z large
	// enough to cover the maximum bevel depth.
	Point3				mExt;
	float				mDepth;

	// Tests
	math::LinePlane		mLP;
	math::SegPlane		mSP;

	math::Order			mOrder;

	Mask				mMask;

	// Tmp storage
	Point3				mPtA;
	Segment3			mSegA;
	Polygon				mPolyB,
						mPolyPrj;		// The destination poly projected onto the source plane
	QRectF				mFrameA, mFrameB;
	RecycleArray<int>	mI;

	// Create the polygon intersection between the two tris, and add it.
	void				addClipper(const Source& a, const Polygon& b);

	// Given a source plane and a comparison poly, find the polygon
	// describing where that poly intersects the plane.  Do this by
	// first finding a new poly on the source plane, and then examining
	// three basic cases:
	// 1)  The comparison poly is completely below the new one.  In this case,
	// use the entire comparison poly as the mask.
	// 2)  The comparison poly is completely above the new one.  In this case,
	// stop, no masking has occured.
	// 3)  The comparison poly has some points above or below the new one.  In
	// this case, find where the poly intersects the destination plane.
	bool				getClipper(const Plane& inP, const Polygon& org, Polygon& clp);
};

} // namespace bevel

#endif // BEVEL_CLIP_H
