#ifndef BEVEL_EDGE_H
#define BEVEL_EDGE_H

#include "collections/managedlist.h"
#include "math/primitives/point3.h"
#include "math/tests/ptseg.h"

namespace bevel {
class Source;

/* BEVEL::EDGE-POINT
 * A single point on an edge of a polygon, including normal info.
 ******************************************************************/
class EdgePoint
{
public:
	static const int	NO_TYPE = 0;
	static const int	ENDPOINT_TYPE = (1<<0);
	// Special type used during building a single polygon source,
	// to prevent overlapping points.
	static const int	BUILD_TYPE = (1<<10);

public:
	EdgePoint();

	// We should just get rid of the loc, and have edges flattened down
	// to distance values, but right now everything down stream is built
	// on it so I'll leave it along at the moment.
	Point3				mLoc;
	Point3				mNrm;
	double				mD;
	// Let the client now if this is part of an edge segment (i.e. connected
	// to the next point in the line).  If it's not, client can do special
	// processing on single points.
	bool				mIsSegment;
	// Temporary during building, the number of points that have been added to this one.
	int					mType;
	int					mCount;

	void				clear();
	void				set(const EdgePoint&);
	void				start(	const Point3& loc, const Point3& nrm,
								const Point3& origin, const int type,
								const bool isSegment);
	void				add(const EdgePoint&);
};

/* BEVEL::EDGE-POINT-LIST
 ******************************************************************/
class EdgePointList : public ManagedList<EdgePoint*>
{
public:
	EdgePointList();

	void				add(EdgePoint&);

private:
	typedef ManagedList<EdgePoint*>	inherited;

// internal
public:
	class Factory;
};

/* BEVEL::EDGE
 * A single edge on a polygon, including normal info.  Because
 * I store normals, there might be multiple Edges for a single
 * polygon edge.
 ******************************************************************/
class Edge
{
public:
	Edge();

	Point3			mA, mB;
	const Source	*mSource;

	EdgePointList	mPts;

	void			clear();
	void			set(const Point3& a, const Point3& b, const Source&);
};

/* BEVEL::EDGE-LIST
 ******************************************************************/
class EdgeList : public ManagedList<Edge*>
{
public:
	EdgeList();

	// Begin a new edge, with the a pt distance at 0 and the b pt at 1.
	Edge*			start(const Point3& a, const Point3& b, const Source&);

	void			add(const Point3& a, const Point3& b, const Source&);

private:
	typedef ManagedList<Edge*>	inherited;

// internal
public:
	class Factory;
};

typedef ManagedListIterator<Edge*>	EdgeListIterator;
typedef ManagedListIterator<EdgePoint*> EdgePointIterator;

} // namespace bevel

#endif // BEVEL_EDGE_H
