#ifndef BEVEL_EDGEBUILDER_H
#define BEVEL_EDGEBUILDER_H

#include "math/mathutils.h"
#include "math/tests/segintersection.h"
#include "fx/bevel_edge.h"

namespace bevel {
class Surface;
class SurfaceList;

/* BEVEL::EDGE-BUILDER
 * Do the work of building the normal info for a polygon edge.
 ******************************************************************/
class EdgeBuilder
{
public:
	/* EDGE-BUILDER::STATE
	 * Persistent info for the edge builder algo.
	 **************************************************************/
	class State
	{
	public:
		State();

		math::SegIntersection		mSI;
		Point3						mPtA0, mPtA1;
		EdgePoint					mEdgePt;		// a template for adding
	};

public:
	EdgeBuilder(State&, Surface&);
	~EdgeBuilder();

	void			add(const SurfaceList&);

private:
	State&			mState;
	Surface&		mA;

	// Add every edge of b that overlaps with a.
	void			addEdges(const Surface& b);

	// Initialize all the edges for my surface.
	void			start();
	// Compile all the overlapping edge points.  The compilate is relatively
	// simple, just looking for overlapping points, because segments should
	// only be adjacent, they should never actually overlap (according to the
	// rules of the beveling frontend).
	void			finish();

// internal
public:
	class CombineBuild;
	class CombineFinish;
};

} // namespace bevel

#endif // BEVEL_EDGEBUILDER_H
