#include "bevel_mask.h"

#include "bevel_source.h"
#include "bevel_surface.h"

using namespace bevel;

/* BEVEL::MASK
 ******************************************************************/
Mask::Mask()
{
}

void Mask::start(const float depth)
{
	mClip.clear();
	mFilter.mZ = depth * 0.5;
}

void Mask::addClipper(const Polygon& p)
{
	if (p.getSize() < 3) return;

	try {
		mClip.pushBack((Polygon*)&p);
	} catch (std::exception&) {
	}
}

void Mask::clip(const Polygon& shape, const Source& dst, SurfaceList& output)
{
if (mPrint) {
	qDebug() << "CLIP on " << dst.mPoly;
}
	try {
		mResult0.clear();
		mResult0.pushBack((Polygon*)&(dst.mPoly));
int c = 0;
		PolygonIterator				itC(mClip);
		while (itC.hasValue()) {
//qDebug() << "MASK.CLIP " << c << " against size " << mResult0.size();
c++;
			const Polygon*			src = *itC;
			++itC;

			mResult1.clear();
			PolygonIterator			it0(mResult0);
			while (it0.hasValue()) {
				clip(*(*it0), *src, mResult1);
				++it0;
			}

			mResult0.swap(mResult1);
		}

		// Take the final output in mResult0 and supply it to the client.
		PolygonIterator				it0(mResult0);
		while (it0.hasValue()) {
			const Polygon&			p = **it0;
			++it0;

if (mPrint) qDebug() << "clip result " << p;

//			qDebug() << "isExternal=" << mPE.on(shape, p);
//			qDebug() << "isExternal=" << isExternal(shape, p);
			if (!isExternal(shape, p)) output.add(dst, p);
		}
	} catch (std::exception&) {
	}
}

void Mask::clip(const Polygon& dst, const Polygon& src, PolygonList& output)
{
	const int		err = mBool.subtract(dst, src, mFilter);
	if (err <= math::Bool::ERROR) {
		output.pushBack((Polygon*)&dst);
		return;
	}

	PolygonIterator			it(mBool.results());
	while (it.hasValue()) {
		const Polygon*		p = *it;
		++it;
//qDebug() << "\t\tBOOL output size=" << p->getSize() << " " << (*p);
		if (p != NULL) output.pushBack((Polygon*)p);
	}
}

bool Mask::isExternal(const Polygon& shape, const Polygon& b)
{
	b.getBarycenter(mBary);
	return !shape.contains(mBary);
}

/* POLY-FILTER-Z
 ******************************************************************/
Mask::PolyFilterZ::PolyFilterZ()
	: mZ(-1)
{
}

bool Mask::PolyFilterZ::accept(const Polygon& p)
{
	// If any value is below the z, then accept, otherwise reject.
	const qint32			size = p.getSize();
	const Point3*			pts = p.getPoints();
	for (int k=0; k<size; k++) {
		if (pts[k].z <= mZ) return true;
	}
	return false;
}
