#ifndef BEVEL_MASK_H
#define BEVEL_MASK_H

#include "math/primitives/polygon.h"
#include "math/poly/bool.h"
#include "math/poly/polyfilter.h"
#include "math/poly/polygonlist.h"
#include "math/tests/polyisexternal.h"

namespace bevel {
class Source;
class SurfaceList;

/* BEVEL::MASK
 * Clip a single polygon against a series of subtraction polys.
 * It's assumed all polys are on the same plane.  Use this class
 * by calling start(), then addClipper() with all subtraction
 * polygons, then finally clip() with the single poly that all
 * the clips will be subtracted from.
 ******************************************************************/
class Mask
{
public:
	bool			mPrint;

public:
	Mask();

	void					start(const float depth);
	// Add a polygon to subtract from my destination.
	void					addClipper(const Polygon&);
	// Clip supplied poly against every poly added with addClipper().
	void					clip(const Polygon& shape, const Source&, SurfaceList& output);

private:
	PolygonList				mClip;
	PolygonList				mResult0, mResult1;

	Polygon					mPolyA, mPolyC;
	Point3					mBary;

	math::Bool				mBool;
	math::PolyIsExternal	mPE;

	// Clip dst against src s, placing the results, in any, in the output.
	void					clip(const Polygon& dst, const Polygon& src, PolygonList& output);

	// Answer true if the poly lies outside the source shape
	bool					isExternal(const Polygon& shape, const Polygon&);

private:
	/* POLY-FILTER-Z
	 * Skip polygons with all points above a given z, I THINK this
	 * is a safe way to eliminate clipped areas.
	 **************************************************************/
	class PolyFilterZ : public math::PolyFilter {
	public:
		PolyFilterZ();
		float				mZ;

		virtual bool		accept(const Polygon&);
	};

	PolyFilterZ				mFilter;
};

} // namespace bevel

#endif // BEVEL_MASK_H
