#include "bevel_source.h"

using namespace bevel;

/* BEVEL::SOURCE
 ******************************************************************/
Source::Source()
{
}

void Source::clear()
{
	mPoly.clear();
}

/* BEVEL::SOURCE-LIST
 ******************************************************************/
class SourceList::Factory : public ManagedFactory<Source*>
{
public:
	virtual Source*		newObject() const {
		Source*			ans = new Source();
		if (ans == NULL) throw std::exception();
		return ans;
	}

	virtual void		deleteObject(Source* o) const { delete o; }
	virtual void		initializeObject(Source* o) const { o->clear(); }
	virtual void		setObject(Source*, Source*) const { throw std::exception(); }
};
static SourceList::Factory	SOURCE_LIST_FACTORY;

SourceList::SourceList()
	: inherited(SOURCE_LIST_FACTORY)
{
}

void SourceList::add(	const Point3* a, const Point3* b, const Point3* c, const Point3* d,
						math::Order& order)
{
	Source*			p = pushBack();
	if (p == NULL) return;
	if (a != NULL) p->mPoly.add(*a);
	if (b != NULL) p->mPoly.add(*b);
	if (c != NULL) p->mPoly.add(*c);
	if (d != NULL) p->mPoly.add(*d);
	order.makeCcw(p->mPoly.getPoints(), p->mPoly.getSize());
	p->mPlane.from(p->mPoly);
}

/* BEVEL::SOURCE-NORMAL
 ******************************************************************/
SourceNormal::SourceNormal()
	: mMaxD(10)
{
	// Hopefully prevents allocations.
	mFound.get(32);
}

void SourceNormal::setMaxDistance(const double d)
{
//qDebug() << "Set depth=" << d;
	mMaxD = d;
//mMaxD = 20;
}

bool SourceNormal::getNormal(const SourceList& all, const Point3& loc, Point3& nrm)
{
	if (mPrint) qDebug() << "\ngetNormal for " << loc << " maxD=" << mMaxD;
//	if (mPrint) qDebug() << "\tstart nrm=" << nrm;

	// Construct a segment that will z-intersect every source in the list
	mSegA0.set(loc);
	mSegA1.set(loc);
	mSegA1.z = mMaxD*100;
	mSegA0.z = -mSegA1.z;

	// Collect all edges within range
	Found*						found = NULL;
	int							size = 0;
	SourceListIterator			it(all);
	while (it.hasValue()) {
		const Source&			s = **it;
		++it;

if (mPrint) qDebug() << "\tcheck poly=" << s.mPoly;
		// Find the intersection
		if (s.mFrame.contains(mSegA0) && s.mPoly.contains(mSegA0)
				&& mLP.on(mSegA0, mSegA1, s.mPlane, &mHit) >= math::LinePlane::INTERSECT) {
if (mPrint) qDebug() << "\t\thit";
			if ((found = mFound.get(size+1)) == NULL) return false;
			Found&				f = found[size];
			f.mD = mHit.z;
			f.mNrm.set(s.mNormal);
			size++;
		}
	}
	if (size < 1) return false;
	if (size == 1) {
		nrm.set(found[0].mNrm);
		return true;
	}
	double					lowD;
	assignAmounts(found, size, lowD);
	normalize(found, size);
	// Accumulate the averages
	nrm.set(0, 0, 0);
	for (int k=0; k<size; k++) {
		Found&			f = found[k];
		f.mNrm *= f.mAmt;
		nrm.asAdd(nrm, f.mNrm);
	}

	return true;
}

void SourceNormal::assignAmounts(Found* found, const int size, double& lowD) const
{
	for (int k=0; k<size; k++) {
		Found&			f = found[k];
		if (f.mD >= mMaxD) f.mAmt = 0;
		else f.mAmt = 1-(f.mD/mMaxD);
	}
}

void SourceNormal::normalize(Found* found, const int size) const
{
	double					sum = 0;
	for (int k=0; k<size; k++) sum += found[k].mAmt;
	// This would be odd, but equal everyone
	if (sum <= 0) {
		sum = 1.0/(double)size;
		for (int k=0; k<size; k++) found[k].mAmt = sum;
	} else {
		for (int k=0; k<size; k++) found[k].mAmt /= sum;
	}
}
