#ifndef BEVEL_SOURCE_H
#define BEVEL_SOURCE_H

#include "collections/managedlist.h"
#include "math/mathutils.h"
#include "math/primitives/plane.h"
#include "math/primitives/polygon.h"
#include "math/primitives/rect.h"
#include "math/tests/lineplane.h"

namespace bevel {

/* BEVEL::SOURCE
 * An extruded edge from the original polygon, along with its
 * center line.
 ******************************************************************/
class Source
{
public:
	Source();

	Point3				mNormal;
	Polygon				mPoly;
	// cached aspects of the poly
	Plane				mPlane;
	math::Rect			mFrame;

	// The previous edge lines, will be identical to the corresponding
	// lines on my prv and nxt sources.
	Segment3			mEdgePrv, mEdgeNxt;

	void				clear();
};

/* BEVEL::SOURCE-LIST
 ******************************************************************/
class SourceList : public ManagedList<Source*>
{
public:
	SourceList();

	// A convenience for adding a new polygon (can be a tri or quad)
	void				add(const Point3*, const Point3*, const Point3*, const Point3*,
							math::Order&);

private:
	typedef ManagedList<Source*>	inherited;

// internal
public:
	class Factory;
};

/* BEVEL::SOURCE-NORMAL
 * A utility for finding the normal of a point on a source list.
 ******************************************************************/
class SourceNormal
{
public:
	bool				mPrint;		// tmp debug

public:
	SourceNormal();

	// Set the maximum distance to shade
	void				setMaxDistance(const double);
	bool				getNormal(const SourceList&, const Point3& loc, Point3& nrm);

private:
	/* FOUND
	 **************************************************************/
	struct Found {
		double			mD;			// distance from 0
		double			mAmt;		// amount I will contribute to final output
		Point3			mNrm;
	};
	typedef RecycleArray<Found>	FoundBlock;

private:
	double				mMaxD;
	Point3				mSegA0, mSegA1, mHit;
	FoundBlock			mFound;

	math::LinePlane		mLP;

	void				assignAmounts(Found*, const int size, double& lowD) const;
	void				normalize(Found*, const int size) const;
};

typedef ManagedListIterator<Source*>	SourceListIterator;

} // namespace bevel

#endif // BEVEL_SOURCE_H
