#ifndef CHAIN_H
#define CHAIN_H

#include <QList>
#include "engine/fx.h"
#include "model/context.h"
#include "engine/fxio.h"

namespace fx {

/* FX::CHAIN
 * A chain does no compositing, it simply
 * runs FX in a sequence.
 ******************************************************************/
class Chain : public Fx
			, public fx::Reader::Target
{
public:
	Chain();
	virtual ~Chain();

	virtual Chain*			copy() const;

	virtual void			init(fx::Setup&);

	virtual QString			getLabel() const;

	virtual void			getParams(param::List&, param::GetArgs&) const;
	virtual param::Parent*	findParent(param::UrlIterator&);
	virtual void			setParam(const param::UrlIterator&, param::SetArgs&);

	virtual int				forEach(Visitor&, VisitParams&);
	virtual int				edit(fx::EditOp&);
	virtual void			work(fx::WorkArgs&, fx::WorkOutput&);

	// IO
	virtual void			writeJson(json::Writer&) const;
	virtual void			startArray(json::Stack&, const QString&);
	virtual void			parseFx(Fx*);

	void					append(Fx*);

protected:
	void					remove(const int index);

private:
	Chain(const Chain&);
	QList<Fx*>				mFx;
};

/* FX::CHAIN-FACTORY
 ******************************************************************/
class ChainFactory : public FxFactory
{
public:
	ChainFactory();

	virtual QString			label() const;
	virtual QString			key() const;
	virtual float			match(const QString& key) const;
	virtual Fx*				create() const;
};

} // fx

#endif // CHAIN_H
