#ifndef GLOW_H
#define GLOW_H

#include "engine/arcdefs.h"
#include "engine/archead.h"
#include "engine/fx.h"
#include "model/context.h"
#include "compositing/workmasks.h"

namespace fx {

/* FX::GLOW
 * Perform a glow operation based on distance to the shape.  Users
 * can extend the glow inside the shape, outside it, or both, and
 * use an arc to shade it as desired.
 ******************************************************************/
class Glow : public Fx
{
public:
	Glow();

	virtual Glow*			copy() const;

	virtual void			init(fx::Setup&);

	virtual QString			getLabel() const;

	virtual void			getParams(param::List&, param::GetArgs&) const;
	virtual param::Parent*	findParent(param::UrlIterator&);
	virtual void			setParam(const param::UrlIterator&, param::SetArgs&);

	virtual void			work(fx::WorkArgs&, fx::WorkOutput&);

	// IO
	virtual void			writeJson(json::Writer&) const;
	virtual void			startObject(json::Stack&, const QString&);
	virtual void			jsonFloat(json::Stack&, const QString&, const float);

private:
	Glow(const Glow&);

	// Amount in pixels to extends the glow from the outline.
	// Amounts at or beyond the limit mean "infinity," as in,
	// take the current min/max distance values and clip to that.
	float					mIn, mOut;
	ColorSrcHead			mClr;
	ArcHead					mArc;

	// Caches
	arc::WorkParams			mArcP;
	WorkMasks				mWorkMasks;
};

/* FX::GLOW-FACTORY
 ******************************************************************/
class GlowFactory : public FxFactory
{
public:
	GlowFactory();

	virtual QString			label() const;
	virtual QString			key() const;
	virtual float			match(const QString& key) const;
	virtual Fx*				create() const;
};

} // fx

#endif // GLOW_H
