#ifndef LIGHT_H
#define LIGHT_H

#include "math/primitives/point3.h"
#include "engine/arcdefs.h"
#include "engine/archead.h"
#include "engine/colorsrchead.h"
#include "engine/fx.h"
#include "model/context.h"
#include "compositing/mask.h"

class Arc;

namespace fx {

/* FX::LIGHT
 * Given a geometry, fill in the frame Cells.
 * While this is a full FX, it's currently
 * implemented and only used as part of the
 * Bevel fx.
 ******************************************************************/
class Light : public Fx
{
public:
	Light();
	explicit Light(Context&);
	Light(const Light&);
	virtual ~Light();

	virtual Light*			copy() const;

	virtual void			init(fx::Setup&);

	void					setLight(	const float bevelDepth, const Point3& loc,
										const float min, const float max);

	virtual QString			getLabel() const;

	virtual void			getParams(param::List&, param::GetArgs&) const;
	virtual param::Parent*	findParent(param::UrlIterator&);
	virtual void			setParam(const param::UrlIterator&, param::SetArgs&);

	virtual void			work(fx::WorkArgs&, fx::WorkOutput&);

	virtual void			writeJson(json::Writer&) const;

private:
	// State
	Point3					mLightLoc;
	float					mBevelDepth, mLightMin, mLightMax;

	// Need to work out whether this is a separate FX or not,
	// right now it's embedded in the bevel and causing API issues.
public:
	ColorSrcHead			mClr;
	ArcHead					mArc;

private:
	// Work support
	arc::WorkParams			mArcP;
	Mask					mRowV, mRowMask;
};

} // fx

#endif // LIGHT_H
