#ifndef SHAPE_H
#define SHAPE_H

#include "engine/fx.h"
#include "model/context.h"

namespace fx {

/* FX::SHAPE
 * Generate an input polygon.  Generally the first FX in a layer.
 ******************************************************************/
class Shape : public Fx
{
public:
	Shape(const QString* filename = NULL);

	virtual Shape*			copy() const;

	virtual void			init(fx::Setup&);

	virtual QString			getLabel() const;

	virtual void			getParams(param::List&, param::GetArgs&) const;
	virtual param::Parent*	findParent(param::UrlIterator&);
	virtual void			setParam(const param::UrlIterator&, param::SetArgs&);

	virtual void			work(fx::WorkArgs&, fx::WorkOutput&);

	// IO
	virtual void			writeJson(json::Writer&) const;
	virtual void			startObject(json::Stack&, const QString&);
	virtual void			jsonString(json::Stack&, const QString&, const QString&);
	virtual void			jsonFloat(json::Stack&, const QString&, const float);

private:
	Shape(const Shape&);

	QString					mFilename;
	QRectF					mFrame;
	ColorSrcHead			mClr;
	bool					mNeedsCache;
	// The root directory for factory shapes
	QString					mShapeDir;
};

/* FX::SHAPE-FACTORY
 ******************************************************************/
class ShapeFactory : public FxFactory
{
public:
	ShapeFactory();

	virtual QString			label() const;
	virtual QString			key() const;
	virtual float			match(const QString& key) const;
	virtual Fx*				create() const;
};

} // fx

#endif // SHAPE_H
