#include "colorbrowser.h"

#include <QPixmap>
#include "model/context.h"
#include "compositing/colorsrclist.h"
#include "engine/fxdefs.h"
#include "fxui/colorsrcrender.h"

/* COLOR-BROWSER::RENDER
 * The visitor that renders each color source in the list and
 * adds the results to the browser
 ******************************************************************/
class ColorBrowser::Render : public ColorSrcList::Visitor
{
public:
	Render(QList<QPixmap*>& e, const QSize& size, const QString& key, int& idx)
		: mE(e), mSize(size), mKey(key), mIdx(idx), mCount(0)		{ }

	virtual int			visit(const ColorSrcHead& h) {
		QPixmap*		pm = mR.onRender(h, mSize);
		if (pm != NULL) {
			mE.append(pm);
			if (mKey.length() > 0 && h.getKey() == mKey) mIdx = mCount;
			mCount++;
		}
		return fx::CONTINUE;
	}

private:
	QList<QPixmap*>&	mE;
	const QSize			mSize;
	const QString&		mKey;
	int&				mIdx;
	ColorSrcRender		mR;
	int					mCount;
};

/* COLOR-BROWSER
 ******************************************************************/
ColorBrowser::ColorBrowser(Context& prj, const ColorSrcList& cl,
							const Param& p, const QSize& iconSize)
	: mPrj(prj)
	, mClrs(cl)
	, mIconSize(iconSize)
	, mIdx(0)
{
	mUrl = p.mUrl;
	Render			r(mE, iconSize, p.mSV, mIdx);
	cl.forEach(r);
}

ColorBrowser::~ColorBrowser()
{
	while (!mE.isEmpty()) delete mE.takeFirst();
}

int ColorBrowser::size() const
{
	return mE.size();
}

QSize ColorBrowser::iconSize() const
{
	return mIconSize;
}

const QPixmap* ColorBrowser::at(const int idx)
{
	if (idx < 0 || idx >= mE.size()) return NULL;
	return mE[idx];
}


int ColorBrowser::getIndex() const
{
	return mIdx;
}

void ColorBrowser::setIndex(const int idx)
{
	const ColorSrcHead*		h = mClrs.getAt(idx);
	if (h != NULL) {
		mPrj.setParamClr(mUrl, *h);
		mIdx = idx;
	}
}
