#ifndef SHAPEBROWSER_H
#define SHAPEBROWSER_H

#include <QPixmap>
#include "engine/fxuidefs.h"
#include "param/param.h"
#include "collections/recyclelist.h"

class Environment;
class Context;
class Look;

namespace fxui {

/* SHAPE-BROWSER.  A model for selecting an item from a list of
 * shapes.
 ******************************************************************/
class ShapeBrowser : public fxui::BrowserModel
{
public:
	ShapeBrowser(	Context&, const Param& p, const QSize& iconSize,
					const Environment&, const Look& look);
	virtual ~ShapeBrowser();

	virtual int				size() const;
	virtual QSize			iconSize() const;

	virtual const QPixmap*	at(const int idx);

	virtual int				getIndex() const;
	virtual void			setIndex(const int idx);

private:
	class Render;
	ShapeBrowser(const ShapeBrowser&);

	Context&				mPrj;
	param::Url				mUrl;
	const QSize				mIconSize;
	int						mIdx;

	struct entry {
		QString				mFn;
		QPixmap				mBm;
	};
	QList<entry*>			mE;

	void					clear();

	const entry*			getAt(const int idx) const;

	void					cacheAll(const Environment&, const Look&);
	entry*					newEntry(const QString& fullFn, const QString& fn, const Look&) const;
};

} // namespace fxui

#endif // SHAPEBROWSER_H
