#ifndef JSON_PARSER_H
#define JSON_PARSER_H

#include <QString>
#include "collections/managedlist.h"
#include "io/json_reader.h"
#include "io/json_stack.h"

class QIODevice;

namespace json {

class State;
class StateFactory;

/* JSON::PARSER
 * A callback-based parser for a JSON-formatted stream.
 * For an explanation of JSON, see: http://www.json.org/
 *
 * Start by installing a Reader, which receives the json data.
 * The parser keeps a stack of the state (in object, in array),
 * and readers can at any time push on a new reader to receive
 * the current data.
 ******************************************************************/
class Parser : public Stack
{
public:
	Parser();
	~Parser();

	bool						parse(QIODevice*, Reader*);

	virtual bool				push(Reader*);

private:
	ManagedList<State*>			mState;			// The state stack.  A new state is
								// pushed by composite json tokens (object, array),
								// as well as new Reader objects.
	Reader*						mCur;			// The current reader
	QString						mKey, mValue;
	bool						mHasSeparator;
	// For parsing in everything not handled atomically,
	// which currently just means numbers.
	const QChar*				mValueStart;

	bool						handle(const QString& src);
	void						flushValue(const QChar* end);

	void						clear();

	// Push a new token-based node on the stack.
	bool						pushToken(const int);

	// Pop up to and including the first node with token
	bool						popTo(const int token);
	bool						removeTo(const int token);

	// Assign the current reader based on the most recent available.
	bool						getCurrentReader();
};

} // namespace json

#endif // JSON_PARSER_H
