#include "svgreader.h"

#include <QtDebug>

static const QString		SVG_D = "d";
static const QString		SVG_PATH = "path";
static const QString		SVG_RECT = "rect";

SvgReader::SvgReader()
{
}

bool SvgReader::load(const QString& filename, QPainterPath* path)
{
	if (path == NULL) return false;

	Handler				h;
	QXmlSimpleReader	reader;
	reader.setContentHandler(&h);
	reader.setErrorHandler(&h);

	QFile				file(filename);
	QXmlInputSource		input(&file);
	const bool			ans = reader.parse(&input, false);
	if (ans && path != NULL) *path = h.mPath;
	return ans;
}

SvgReader::Handler::Handler()
	: mCmdC(mPath, mLastPt)
	, mCmdL(mPath, mLastPt)
	, mCmdM(mPath, mLastPt)
{
	mBuf.reserve(256);
}

bool SvgReader::Handler::startElement(	const QString & namespaceURI,
										const QString & localName,
										const QString & qName,
										const QXmlAttributes & atts)
{
	const bool		ans = inherited::startElement(namespaceURI, localName, qName, atts);
//	qDebug() << "startElement " << namespaceURI << " " << localName << " " << qName;
	if (localName == SVG_PATH) {
		const QString	d = atts.value(SVG_D);
//		qDebug() << "\td=" + d;
		if (d.size() > 0) addPath(d);
	} else if (localName == SVG_RECT) {
		const float		x = atts.value("x").toFloat(), y = atts.value("y").toFloat(),
						w = atts.value("width").toFloat(), h = atts.value("height").toFloat();
		mPath.addRect(x, y, w, h);
	}
	return ans;
}

void SvgReader::Handler::addPath(const QString& d) {
	mLastPt.setX(0);
	mLastPt.setY(0);

	SvgCmd*					last = NULL;
	mBuf.resize(0);

	for (int k=0; k<d.size(); k++) {
		const QChar			c = d.at(k);

		// MOVETO.  Not sure if this is a bug with Inkscape, but
		// sometimes it stacks up movetos like linetos, so for now
		// I'll convert any subsequent M's to L's.
		if (c == 'm') {
			last = startCmd(last, &mCmdM, true);
		} else if (c == 'M') {
			last = startCmd(last, &mCmdM, false);
		// LINETO
		} else if (c == 'l') {
			last = startCmd(last, &mCmdL, true);
		} else if (c == 'L') {
			last = startCmd(last, &mCmdL, false);
		//  CUBIC BEZIER
		} else if (c == 'c') {
			last = startCmd(last, &mCmdC, true);
		} else if (c == 'C') {
			last = startCmd(last, &mCmdC, false);
/*
		// ARC
		} else if (c == 'a') {
			last = startCmd(last, mCmdA, true);
		} else if (c == 'A') {
			last = startCmd(last, mCmdA, false);
*/
		// CLOSE
		} else if (c == 'z' || c == 'Z') {
			last = NULL;
//			mPath.close();
//			if (PRINT) System.out.println("CLOSE");

		// deal with the pt x,y separator (WHY didn't they just
		// have arguments be flat lists of numbers, REALLY?
		} else if (c == ',') {
			flushBuffer(last);

		// handle white space by flushing my buffer
		} else if (c.isSpace()) {
			flushBuffer(last);

			// Everything else is parsing values
		} else {
			mBuf.append(c);
		}
	}
}

SvgCmd* SvgReader::Handler::startCmd(SvgCmd* prev, SvgCmd* next, const bool isRelative)
{
	flushBuffer(prev);
	if (next != NULL) {
		next->start(isRelative);
	}
	return next;
}

void SvgReader::Handler::flushBuffer(SvgCmd* cmd) {
	// Convert the buffer to a float and handle it.
	if (mBuf.size() > 0) {
		if (cmd != NULL) {
			bool			ok;
			const float		v = mBuf.toFloat(&ok);
			if (ok) cmd->push(v);
		}
		mBuf.resize(0);
	}
}
