#include <QtDebug>
#include <QtGui/QApplication>
#include "arcwindow.h"
#include "app/defs.h"
#include "app/win.h"
#include "app/environment.h"
#include "compositing/colorsrclist.h"
#include "model/activeproject.h"
#include "compositing/mask.h"

#include "fx/bevel.h"
#include "fx/chain.h"
#include "fx/glow.h"
#include "fx/shape.h"

#include "experimental/exp_defs.h"

int main(int argc, char *argv[])
{
	// Client initialization
	init_mask();

	// All arcs in the system.
	ArcList				arcs;
	ArcList::addDefaults(arcs);
	// All colorsrcs in the system
	ColorSrcList		cs;
	ColorSrcList::addDefaults(cs);

	// Setup the fx plugin info.
	FxFactory::add(new fx::BevelFactory());
	FxFactory::add(new fx::ChainFactory());
	FxFactory::add(new fx::GlowFactory());
	FxFactory::add(new fx::ShapeFactory());

	// Run the application.
	QApplication		a(argc, argv);
	Environment			e(a);
	ActiveProject		p(arcs, cs, e);

	// Load the default project
	{
		QString			projFn = e.getInstallPath(PROJECTS_INSTALL_DIR);
		projFn = projFn + QDir::separator() + "init.json";
		QFile			f(projFn);
		if (f.open(QFile::ReadOnly)) p.readJson(&f);
	}
	p.verify();

	// Initially using QtDesigner for the top-level GUI, but I'm
	// not sure how to add custom classes to it.  On investigating
	// it looked a little painful -- like you have to write a plugin
	// that supplies class info.  Maybe I'm spoiled from Android,
	// where you just use your scoped custom class name.
//    ArcWindow w;
//    w.show();
	Win					win(e, arcs, cs, p);
	win.show();

	const int			ans = a.exec();

	FxFactory::shutdown();
	return ans;
}

/* TESTING
 * A few initialization functions to help with testing
 ******************************************************************/
#if 0

#include "io/json_writer.h"

static Fx* newFx(const Environment& e)
{
	fx::Chain*			c = new fx::Chain();
	const QString		root = SHAPES_DIR + QDir::separator();
	QString				fn;

//	fn = root + "circle-256.svg";
//	fn = root + "circle-32.svg";
//	fn = root + "semi.svg";
//	fn = root + "phone_speaker.svg";
	fn = root + "is_penta.svg";

	if (c != NULL) {
		c->append(new fx::Shape(&fn));
		c->append(new fx::Bevel());
		c->append(new fx::Glow());
	}
	return c;
}

static void testJsonWriter(ActiveProject& prj, const QString& fn)
{
	qDebug() << "WRITE test json=" << fn;
	{
		QFile					f(fn);
		if (f.open(QFile::WriteOnly | QFile::Truncate)) {
			json::PrettyFormat	fmt;
			json::Writer		w(&f, fmt);
			prj.writeJson(w);
		}
	}
	{
		QFile					f(fn);
		if (f.open(QFile::ReadOnly)) {
			prj.readJson(&f);
		}
	}
}

static void testInit(ActiveProject& prj, const Environment& env)
{
	Fx*					fx = newFx(env);
	if (fx != NULL) {
		fx->init(prj);
		p.setFx(fx);
	}
	// Test file writing
	{
		QString			path(env.getInstallPath());
		path.append(QDir::separator());
		path.append("fx.json");
		testJsonWriter(prj, path);
	}
}

#endif
