#ifndef MATHUTILS_H
#define MATHUTILS_H

#include "collections/recyclearray.h"
#include "math/primitives/point3.h"
class Polygon;

namespace math {

/* MATH::ORDER
 * A utility for ordering points.
 ******************************************************************/
class Order {
public:
	Order();

	void					makeCcw(Point3* pts, const int size);

private:
	RecycleArray<double>	mD;

	bool					verify(const Point3* pts, const int size);
	void					cacheDegrees(const Point3* pts, const int size);
};

/* MATH::NORMAL
 * Calculate a surface normal.  All points must be on the same plane.
 ******************************************************************/
struct Normal {
	bool					setOn(const Polygon&, Point3& nrm);

private:
	Point3					mA, mB;
};

/* MATH::EDGE-ITERATOR
 * A utility for iterating over the edges of a polygon.
 ******************************************************************/
class EdgeIterator {
public:
	EdgeIterator(const Polygon&);

	const Point3			*mA, *mB;

	void					operator++();

	bool					hasValue() const;

private:
	const qint32			mSize;
	const Point3*			mPts;
	qint32					mIdx;
};

} // namespace math

#endif // MATHUTILS_H
