#include "bool.h"

#include "math/poly/boolimg.h"
#include "math/poly/polyfilter.h"

using namespace math;

/* BOOL
 ******************************************************************/
Bool::Bool()
	: mEvilA(mPP)
{
}

int Bool::subtract(const Polygon& dst, const Polygon& src)
{
	PolyFilter				f;
	return subtract(dst, src, f);
}

int Bool::subtract(const Polygon& dst, const Polygon& src, PolyFilter& f)
{
	try {
		return doSubtract(dst, src, mResults, f);
	} catch (std::exception&) {
	}
	return ERROR;
}

PolygonIterator Bool::results() const
{
	return PolygonIterator(mResults);
}

static bool	IMG = true;
int Bool::doSubtract(	const Polygon& dst, const Polygon& src,
						PolygonList& results, PolyFilter& f)
{
if (PRINT) qDebug() << "\n\nPOLY SUBTRACTION";
	results.clear();
	if (!init(dst, src)) return ERROR;
	const int		rootCount = setAlignment(mDst, src, false);
	setAlignment(mSrc, dst, true);

if (PRINT) {
	qDebug() << "DST " << dst;
	qDebug() << "SRC " << src;
	qDebug() << "DST steps";
	mDst.print();
	qDebug() << "SRC steps";
	mSrc.print();
	qDebug() << "roots=" << rootCount;

	if (IMG) {
		BoolImg			bi;
		bi.dump(mDst, mSrc, 4, "/Users/klarise/Pictures/bool.png");
		IMG = false;
	}
}
	if (rootCount < 1) {
		// If there are no roots, then dst is completely
		// covered by src, so we answer an empty set.
		// XXX This is not necessarily the case -- it could be that the
		// clip polygon clips out all the dst points, but there are
		// still valid edges.  Which implies that I should be using
		// a test of "GOOD" edge "NEUTRAL" point as my starting
		// condition.
		return OK;
	}
	const int		intersectionCount = mDst.intersectionCount();
if (PRINT) qDebug() << "intersections=" << intersectionCount;
	if (intersectionCount < 1) {
		// No intersections could mean one of two things:
		// Either there is no overlap, or src is completely
		// covered by dst.  We still need to implement that
		// second case by removing the "hole" dst creates.
		// If any point from src is inside dst, then dst completely
		// encompasses src, and a hole is created.
		const Point3&		pt = src.getPoints()[0];
		if (dst.contains(pt)) {
			Polygon*		ans0 = results.pushBack();
			Polygon*		ans1 = results.pushBack();
			if (ans0 == NULL || ans1 == NULL) return ERROR;
if (PRINT) qDebug() << "Output cutout";
			if (mCutOut.on(dst, src, *ans0, *ans1)) return OK;
			results.clear();
		}
		return ERROR;
	}

	// We can hit a case where two polygons are aligned along
	// a single edge.  No clipping occurs.
	if (mPIE.on(dst, src)) {
		if (PRINT) qDebug() << "poly is external";
/*
qDebug() << "isExternal";
qDebug() << "DST=" << dst;
qDebug() << "SRC=" << src;
mPIE.mPrint = true;
mPIE.on(dst, src);
mPIE.mPrint = false;
*/

//BoolImg			bi;
//bi.dump(mDst, mSrc, 4, "/Users/klarise/Pictures/bool.png");
		Polygon*				ans = results.pushBack();
		if (ans == NULL) return ERROR;
		ans->set(dst);
		return OK;
	}

	mEvilA.mPoly = &src;
	// Prepare the edges, which guide the building process
	if (!mEdgeA.start(mDst, true, &mEvilA)) return ERROR;
	if (!mEdgeB.start(mSrc, false, NULL)) return ERROR;
	mBuild.on(mEdgeA, mEdgeB, results, f);
	return OK;
}

bool Bool::init(const Polygon& dst, const Polygon& src)
{
	mFlatten.start();
	mFlatten.setFindIntersectionSource(&src, NULL);
	mDst.flatten(dst, mFlatten);
	mFlatten.setFindIntersectionSource(NULL, &mDst);
	mSrc.flatten(src, mFlatten);
	return true;
}

int Bool::setAlignment(StepList& a, const Polygon& b, const bool insideIsGood)
{
	int					goodCount = 0;
	for (int k=0; k<a.mSize; k++) {
		Step&			s = a.mStep[k];
		if (s.isIntersection()) continue;

		const bool		inside = mPP.on(*(s.mPt), b);
		if (inside) {
			if (insideIsGood) s.mAlignment = Step::GOOD;
			else s.mAlignment = Step::EVIL;
		} else {
			if (insideIsGood) s.mAlignment = Step::EVIL;
			else s.mAlignment = Step::GOOD;
		}
		if (s.mAlignment == Step::GOOD) goodCount++;
	}
	return goodCount;
}

/* MATH::BOOL::EVIL-DOER-A
 **************************************************************/
Bool::EvilDoerA::EvilDoerA(math::PtInPoly& pp)
	: mPoly(NULL)
	, mPP(pp)
{
}

void Bool::EvilDoerA::on(Edge& e)
{
	if (e.mAlignment <= Step::EVIL) return;
	// Take the midpoint of the edge - if it's inside the
	// poly, then the edge is invalid.
	mPt.asMid(*(e.mA->mPt), *(e.mB->mPt));
/*
mPP.mPrint = true;
qDebug() << "MID " << mPt << " inPoly?";
qDebug() << "\t" << mPP.on(mPt, *mPoly);
mPP.mPrint = false;
*/
	if (mPP.on(mPt, *mPoly)) e.mAlignment = Step::EVIL;
}
