#ifndef BOOL_H
#define BOOL_H

#include "math/poly/boolcutout.h"
#include "math/poly/build.h"
#include "math/poly/edge.h"
#include "math/poly/evildoer.h"
#include "math/poly/flatten.h"
#include "math/poly/polygonlist.h"
#include "math/poly/steplist.h"
#include "math/tests/ptinpoly.h"
#include "math/tests/polyisexternal.h"

namespace math {
class PolyFilter;

/* BOOL
 * A boolean operation on two polygons.  While this began as general
 * purpose code, over time it has grown specific to the one operation
 * we use, subtraction.
 *
 * The basic algorithm takes a destination polygon and a source (in
 * our case clipping) poly, categorizes all the points as inside,
 * outside or intersecting the other poly, then builds an edge list
 * for each poly and categorizes the edges based on the points.
 * Finally, it traverses all the good or neutral edges, building
 * a new poly (or multiple polygons).
 ******************************************************************/
class Bool
{
public:
	static const int			OK = 0;
	static const int			ERROR = -1;
	// debug
	static const bool			PRINT = false;

public:
    Bool();

	int							subtract(const Polygon& dst, const Polygon& src);
	int							subtract(const Polygon& dst, const Polygon& src, PolyFilter&);

	PolygonIterator				results() const;

protected:
	friend class BoolCutOut;

	int							doSubtract(const Polygon& dst, const Polygon& src,
										   PolygonList& results, PolyFilter&);

private:
	PolygonList					mResults;

	StepList					mDst, mSrc;
	EdgeList					mEdgeA, mEdgeB;

	Flatten						mFlatten;
	Build						mBuild;
	BoolCutOut					mCutOut;

	math::PolyIsExternal		mPIE;
	math::PtInPoly				mPP;

	bool						init(const Polygon& dst, const Polygon& src);
	// Set the alignment value for each step.  The alignment is neutral
	// on intersections, otherwise it's determined by whether the step
	// point is inside polygon b, and what that means according to the
	// insideIsGood var.
	int							setAlignment(StepList& a, const Polygon& b, const bool insideIsGood);

private:
	/* EVIL-DOER-A
	 * Rules for assigning edge alignments on the A list.
	 **************************************************************/
	class EvilDoerA : public EvilDoer {
	public:
		const Polygon*		mPoly;

	public:
		EvilDoerA(math::PtInPoly&);

		virtual void		on(Edge&);

	private:
		math::PtInPoly&		mPP;
		Point3				mPt;
	};

	EvilDoerA				mEvilA;
};

} // namespace math

#endif // BOOL_H
