#ifndef BOOLCUTOUT_H
#define BOOLCUTOUT_H

#include "math/primitives/segment3.h"
#include "math/poly/polygonlist.h"
#include "math/poly/polyfilter.h"
#include "math/tests/lineline.h"
#include "math/tests/ptseg.h"

namespace math {
class Bool;

/* MATH::BOOL-CUT-OUT
 * A helper class for Bool that takes a dst poly, and
 * a src poly completely inside the dst, and returns a new
 * poly that is a single path including both the dst and
 * src, i.e. the src cut out of the dst.
 ******************************************************************/
class BoolCutOut
{
public:
	BoolCutOut();

	// Given two polygons, where we know b is entirely contained
	// within a, place in results new polygons that are the result
	// of subtracting b from a.
	void				on(	const Polygon& a, const Polygon& b,
							Bool&, PolygonList& results, PolyFilter&);

private:
	Segment3			mBisector;
	Polygon				mFrame;

	bool				getBisector(const Polygon& a, const Polygon& b, Segment3&) const;
	bool				getFrame(const Polygon& a, Polygon&) const;

public:
	bool				on(	const Polygon& dst, const Polygon& src,
							Polygon& out0, Polygon& out1);

private:
	/* HIT
	 **************************************************************/
	struct Hit {
		// Indexes into the dst and src polygon.
		int				mDst, mSrc;
		// The intersection on dst that the src points found
		// as the closest.
		Point3			mPt;

		// Tmp during setup
		double			mD;

		void			clear();

		void			swap(Hit&);
	};

private:

	static const int	HIT_SIZE = 2;
	Hit					mHit[HIT_SIZE];

	math::PtSeg			mPS;
	math::LineLine		mLL;

	// Add half the final poly
	void				addPoly0(	const Polygon& dst, const Polygon& src,
									Polygon& out);
	// Add the other half the final poly
	void				addPoly1(	const Polygon& dst, const Polygon& src,
									Polygon& out);

	void				addPoly(	const Polygon& p, const int start, const int end,
									const int delta, Polygon& out, const bool close);

	bool				getStarts(const Polygon& dst, const Polygon& src);
	// Answer the mHit with the highest mD (-1 for off).
	int					highestD() const;
};

} // namespace math

#endif // BOOLCUTOUT_H
