#ifndef BOOLIMG_H
#define BOOLIMG_H

#include <QBrush>
#include <QColor>
#include <QPen>
#include <QRectF>
#include <QString>
#include "math/primitives/range.h"
class Point3;

namespace math {
class StepList;

/* BOOL-IMG
 * A debug utility for generating a bitmap displaying the two
 * polygons being operated on.
 ******************************************************************/
class BoolImg
{
public:
    BoolImg();

	// Scale is a magnifier, so 2 for 2x the original size.
	void			dump(const StepList& a, const StepList& b,
						 const float scale, const QString& filename);

private:
	// metrics
	QPoint			mOrigin;
	float			mScale;
	float			mBorderL, mBorderT;
	// painting
	QColor			mBg_C, mEdgeA_C, mEdgeB_C, mPtA_C, mPtB_C, mI_C,
					mGood_C, mEvil_C;
	QPen			mPen;
	QBrush			mBrush;
	Range			mEdgeR;

	void			drawEdges(	const StepList& a, const QColor& c,
								QPainter& p);
	void			drawPoints(	const StepList& a, const QColor& c,
								const float first, QPainter& p);
	void			getFrame(const StepList&, QRectF&) const;

	void			translate(const Point3&, QPointF& out) const;
};

} // namespace math

#endif // BOOLIMG_H
