#ifndef BUILD_H
#define BUILD_H

#include "math/poly/steplist.h"
#include "math/poly/polygonlist.h"

namespace math {
// forward declarations
class Edge;
class EdgeList;
class PolyFilter;

/* BUILD
 * Convert two EdgesLists into a new polygon by finding
 * the next step during the build process.
 ******************************************************************/
class Build
{
public:
	Build();

	void				on(EdgeList& a, EdgeList& b, PolygonList&, PolyFilter& f);

private:
	/* SOURCE
	 * A place to store the current edge list info.
	 **************************************************************/
	struct Source {
		EdgeList*		mList;
		// The current index.  Only valid for the "a" source, except in
		// cases where we're peeking ahead.
		int				mIdx;

		void			start(const int idx, EdgeList&);
		Edge*			cur();
		void			step();
		bool			seek(const Intersection*);
	};

private:
	static const bool	PRINT = false;
	static const bool	PRINT_OUTPUT = false;

	Source*				mA;
	Source*				mB;

	Source				mA_Stack, mB_Stack;

	Polygon				mTmpR;

	bool				nextPoly(Polygon& p, EdgeList& a, EdgeList& b);
	// idxA must be a valid index into a mStep.
	void				start(const int idxA, EdgeList& a, EdgeList& b);
	Edge*				nextEdge();
	void				switchSource();
};

} // namespace math

#endif // BUILD_H
