#include "edge.h"

#include "math/poly/evildoer.h"
#include "math/poly/steplist.h"

using namespace math;

/* EDGE
 ******************************************************************/
Edge::Edge()
{
}

void Edge::set(Step& a, Step& b, EvilDoer* evil)
{
	mA = &a;
	mB = &b;
	mAlignment = Step::GOOD;
	if (mA->mAlignment < Step::NEUTRAL || mB->mAlignment < Step::NEUTRAL) mAlignment = Step::EVIL;
	if (evil != NULL) evil->on(*this);
}

/* EDGE-LIST
 ******************************************************************/
EdgeList::EdgeList()
	: mEdge(NULL)
	, mSize(0)
{
	// This should prevent me from ever needing to grow.
	mEdge = get(32);
}

bool EdgeList::start(StepList& a, const bool forward, EvilDoer* evil)
{
	mSize = 0;
	if (a.mSize < 2) return false;
	Edge*			newEdge = get(a.mSize+8);
	if (newEdge == NULL) return false;
	mEdge = newEdge;
	mSize = a.mSize;

	int				first, last, stop, inc;
	if (forward) {
		first = 0;
		last = a.mSize-1;
		stop = a.mSize;
		inc = 1;
	} else {
		first = a.mSize-1;
		last = 0;
		stop = -1;
		inc = -1;
	}

	Step*			prv = &(a.mStep[last]);
	int				prvK = last;
	int				dstK = 0;
	for (int k = first; k != stop; k += inc) {
		Step&		nxt = a.mStep[k];
		mEdge[dstK].set(*prv, nxt, evil);

		prv = &nxt;
		prvK = k;
		dstK++;
	}
	return true;
}

int EdgeList::findIndexForA(const Intersection* inter) const
{
	for (int k=0; k<mSize; k++) {
		if (mEdge[k].mA->mIntersection == inter) return k;
	}
	return -1;
}

/* debug
 ******************************************************************/
void EdgeList::print()
{
	for (int k=0; k<mSize; k++) {
		qDebug() << "\t" << k << (mEdge[k]);
	}
}

QDebug& math::operator<<(QDebug& d, const Edge& o)
{
	(d << "Edge align=").nospace();
	(d << o.mAlignment).nospace();
	if (o.mA != NULL) {
		(d << " a=").nospace();
		(d << *(o.mA)).nospace();
	}
	if (o.mB != NULL) {
		(d << " b=").nospace();
		(d << *(o.mB)).nospace();
	}
	return d;
}
