#ifndef EDGE_H
#define EDGE_H

#include "collections/recyclearray.h"
#include "math/poly/step.h"

namespace math {
class EvilDoer;
class StepList;

/* EDGE
 * A single edge in one of the source polygons, with information
 * about whether it's valid or not.
 ******************************************************************/
class Edge
{
public:
	// I don't own the steps, they are stored in the StepList
	Step			*mA, *mB;
	// Use the alignment values from the Step
	int				mAlignment;

public:
    Edge();

	void			set(Step& a, Step& b, EvilDoer*);
};

QDebug& operator<<(QDebug&, const Edge&);

/* EDGE-LIST
 ******************************************************************/
class EdgeList : public RecycleArray<Edge>
{
public:
	// Clients can directly access my data, but note that it's
	// managed by my superclass
	Edge*			mEdge;
	int				mSize;

public:
	EdgeList();

	// Set myself to the edges of steplist a, using b to determine
	// which edges should be marked as EVIL.  Set forward to false if
	// you want to iterate in the reverse order (this is for lists
	// that should be built backwards).
	bool			start(StepList&, const bool forward, EvilDoer*);

	int				findIndexForA(const Intersection*) const;

	void			print();
};

} // namespace math

#endif // EDGE_H
