#include "flatten.h"

using namespace math;

// Ended up with multiple precisions as I was experimenting
static const double			ADD_EPS = 0.01;
static const double			SS_EPS = 0.0000000001;

/* INTERSECTION-FACTORY
 ******************************************************************/
class IntersectionFactory : public ManagedFactory<Intersection*>
{
public:
	virtual Intersection*	newObject() const {
		Intersection*		ans = new Intersection();
		if (ans == NULL) throw std::exception();
		return ans;
	}

	virtual void		deleteObject(Intersection* p) const {
		delete p;
	}

	virtual void		initializeObject(Intersection*) const {
	}

	virtual void		setObject(Intersection* dst, Intersection* src) const {
		if (src != NULL) dst->set(*src);
	}
};
static IntersectionFactory	INTERSECTION_FACTORY;

/* FLATTEN
 ******************************************************************/
Flatten::Flatten()
	: mEps(ADD_EPS)
	, mIntersections(INTERSECTION_FACTORY)
{
	mSS.mEps = 0.0001f;
}

void Flatten::start()
{
	mIntersections.clear();
}

void Flatten::setFindIntersectionSource(const Polygon* p, const StepList* l)
{
	mPoly = p;
	mList = l;
}

static bool intersectionLessThan(const Intersection* o1, const Intersection* o2)
{
	if (o1->mSortOrigin == NULL) return true;

	const double		d1 = o1->mSortOrigin->dist3Squared(o1->mPt),
						d2 = o1->mSortOrigin->dist3Squared(o2->mPt);
	return d1 < d2;
}

void Flatten::addIntersections(const Point3& a, const Point3& b, StepList& list)
{
	mSort.clear();
	mSortOrigin = &a;

	if (mPoly != NULL) getNewIntersections(a, b);
	else if (mList != NULL) getExistingIntersections(a, b);

	if (mSort.length() > 0) {
		qSort(mSort.begin(), mSort.end(), intersectionLessThan);
		for (int k=0; k<mSort.length(); k++) list.add(mSort.at(k), ADD_EPS);
	}
	mSort.clear();
}

void Flatten::getNewIntersections(const Point3& a, const Point3& b)
{
	const qint32					psize = mPoly->getSize();
	const Point3*					pts = mPoly->getPoints();

	const Point3*					prv = &(pts[psize-1]);
	Point3							nearest;
	for (int k=0; k<psize; k++) {
		const Point3&				nxt = pts[k];
//qDebug() << "INTER BETWEEN between " << a << ", " << b << " and " << *prv << ", " << nxt << " -pre=" << mSS.mEps;
		if (mSS.on(a, b, *prv, nxt, &nearest)) {
//qDebug() << "\thit " << nearest;
			Intersection*			inter = mIntersections.pushBack();
			if (inter != NULL) {
				inter->set(nearest, a, b, *prv, nxt);
				inter->mSortOrigin = mSortOrigin;
				mSort.append(inter);
			}
		}
		prv = &nxt;
	}
}

void Flatten::getExistingIntersections(const Point3& a, const Point3& b)
{
	for (int k=0; k<mList->mSize; k++) {
		const Intersection*			inter = mList->mStep[k].mIntersection;
		if (inter == NULL) continue;
		// The intersection was added with the source segment as a,
		// so I need to check the reverse.
		if ( (&a == inter->mB[0] && &b == inter->mB[1]) || (&a == inter->mB[1] && &b == inter->mB[0])) {
			((Intersection*)inter)->mSortOrigin = mSortOrigin;
			mSort.append(inter);
		}
	}
}
