#ifndef FLATTEN_H
#define FLATTEN_H

#include "collections/managedlist.h"
#include "math/primitives/polygon.h"
#include "math/poly/intersection.h"
#include "math/poly/steplist.h"
#include "math/tests/segseg.h"

namespace math {

/* FLATTEN
 * Build the intersected step lists from the source polygons.
 ******************************************************************/
class Flatten
{
public:
	double						mEps;

public:
    Flatten();

	void						start();
	// Set to one or the other, based on whether you want intersections
	// generated or looked up.
	void						setFindIntersectionSource(const Polygon*, const StepList*);

	void						addIntersections(const Point3& a, const Point3& b, StepList& list);

private:
	ManagedList<Intersection*>	mIntersections;

	// Source object for the intersections.  If it's a poly, then
	// the intersections are generated against the line segments.
	// If it's a StepList, I can just directly look up the existing
	// intersections.
	const Polygon*				mPoly;
	const StepList*				mList;

	// When there are multiple intersections across the same segment,
	// they need to be ordered correctly.
	QList<const Intersection*>	mSort;
	const Point3*				mSortOrigin;

	math::SegSeg				mSS;

	// Find all intersections between the segment and mPoly.
	void						getNewIntersections(const Point3& a, const Point3& b);
	// The intersections have already been created, so just find them.
	void						getExistingIntersections(const Point3& a, const Point3& b);
};

} // namespace math

#endif // FLATTEN_H
