#ifndef STEP_H
#define STEP_H

#include "math/primitives/point3.h"
#include "math/poly/intersection.h"

namespace math {

/* STEP
 * A single step in a flattened point list.
 ******************************************************************/
class Step
{
public:
	// Each step has an alignment of GOOD, NEUTRAL or EVIL.  Neutral
	// is synonymous for being an intersection; Good indicates this point
	// must be in the final output, Evil indicates it can't be.  Good points
	// can only be used once, so once they are, they become Evil.
	static const int		GOOD = 1;
	static const int		NEUTRAL = 0;
	static const int		EVIL = -1;

public:
	// I don't own the point.  It is either owned by the
	// source polygon or the intersection.
	const Point3*			mPt;
	int						mAlignment;
	// My intersection, if any.  I don't own this object.
	const Intersection*		mIntersection;

public:
    Step();

	void					set(const Point3&);
	void					set(const Intersection&);
	bool					isIntersection() const;
};

QDebug& operator<<(QDebug&, const Step&);

} // namespace math

#endif // STEP_H
