#ifndef STEPLIST_H
#define STEPLIST_H

#include "collections/recyclearray.h"
#include "math/primitives/polygon.h"
#include "math/poly/step.h"

namespace math {
class Flatten;

/* STEP-LIST
 * A list of Steps, flattened from an incoming polygon.
 ******************************************************************/
class StepList : public RecycleArray<Step>
{
public:
	// Clients can directly access my data, but note that it's
	// managed by my superclass
	Step*			mStep;
	int				mSize;

public:
	StepList();

	// Add a step for each point in p, as well as any
	// intersections with interP.  If List inter is supplied,
	// then it will contain all intersections.
	void			flatten(const Polygon&, Flatten&);

	// Add the point, provided it's unique from the previous.
	void			add(const Point3& pt, const double eps);
	// Add the point, provided it's unique from the
	// previous.  If it's not, then convert the existing
	// point into an intersection.
	void			add(const Intersection*, const double eps);

	int				intersectionCount() const;

	int				findIndexFor(const Intersection*) const;

	void			print();

private:
	// Answer true if pt is within eps distance of my tail point, if any.
	bool			sameAsLast(const Point3& pt, const double eps) const;
};

} // namespace math

#endif // STEPLIST_H
