#ifndef POINT3_H
#define POINT3_H

#include <QtDebug>

/* POINT-3
 * Point, 3D.
 ******************************************************************/
class Point3
{
public:
    Point3();
	Point3(const Point3&);
	Point3(const double x, const double y, const double z);

	double			x, y, z;

	void			set(const Point3&);
	void			set(const double x, const double y, const double z);

	void			swap(Point3&);

	// Equivalence test
	bool			sameAs2(const Point3&) const;
	bool			sameAs2(const Point3&, const double precision) const;
	bool			sameAs3(const Point3&) const;
	bool			sameAs3(const Point3&, const double precision) const;

	const Point3 operator+(const Point3& rhs) const { return Point3(x+rhs.x, y+rhs.y, z+rhs.z); }
	const Point3 operator-(const Point3& rhs) const { return Point3(x-rhs.x, y-rhs.y, z-rhs.z); }
	Point3& operator+=(const Point3& o) { x += o.x; y += o.y; z += o.z; return *this; }
	Point3& operator*=(const double& v) { x *= v; y *= v; z *= v; return *this; }
	Point3& operator/=(const double& v) { x /= v; y /= v; z /= v; return *this; }

	void normalize();
	double length() const;

	// Distance, 2d and 3d
	double dist2(const Point3&) const;
	double dist3(const Point3&) const;
	double dist3Squared(const Point3&) const;

	// Dot product, 2d and 3d
	double			dot2(const Point3&) const;
	double			dot3(const Point3&) const;

	// Set me to the addition of a and b
	void			asAdd(const Point3& a, const Point3& b);
	// Set me to the subtraction of a and b
	void			asSub(const Point3& a, const Point3& b);
	// Set me to the cross product of a and b, 3d
	void			asCross3(const Point3& a, const Point3& b);
	// Given two points, answer a value between them based
	// on pos.  A pos of 0 is a, 0.5 is the mid point, 1 is b.
	void			asBlend(const Point3& a, const Point3& b, const double pos);
	void			asMid(const Point3& a, const Point3& b);
};

QDebug& operator<<(QDebug&, const Point3&);

#endif // POINT3_H
