#ifndef POLYGON_H
#define POLYGON_H

#include <QtDebug>
#include <QtGlobal>

#include "math/primitives/rect.h"

class QPolygonF;
class QRectF;
class Point3;

/* POLYGON
 ******************************************************************/
class Polygon
{
public:
	Polygon();
	~Polygon();

	qint32			getSize() const;
	Point3*			getPoints() const;

	bool			isEmpty() const;
	void			clear();

	bool			set(const Polygon&);
	bool			set(const QPolygonF&);

	bool			setSize(const qint32 newSize);

	bool			add(const Point3&);
	bool			add(const double x, const double y, const double z);

	// Remove all adjacent points within the given tolerance, i.e.
	// remove what we have loosely identified as "duplicates."
	void			compress(const double eps);

	bool			contains(const Point3&) const;
	// Answer true if the polygon is ordered clockwise, false
	// if it's counter.  Poly can be opened or closed.
	bool			isClockWise() const;
	bool			isClockWise(const double precision) const;

	// Get the frame, 2 and 3 dimensions.  The 3 will result in a 6-side,
	// 8-vertex cube.  For the frame, the first 4 pts are the lowest z
	// (i.e. the bottom of the cube), the 2nd 4 are the top.
	void			getFrame(math::Rect&) const;
	void			getFrame(QRectF&) const;
	bool			getFrame(Polygon&) const;

	void			getBarycenter(Point3& bary) const;

private:
	Point3*			mPt;
	qint32			mSize,
					mAlloc;

	bool			growTo(const qint32 newAlloc);
};

QDebug& operator<<(QDebug&, const Polygon&);

#endif // POLYGON_H
