#ifndef RANGE_H
#define RANGE_H

#include <QtDebug>

class Range;

extern const Range		RANGE_UNI;

/* RANGE
 * Translate between value ranges.
 ******************************************************************/
class Range
{
public:
	double				mMin, mMax;

public:
    Range();
	Range(const Range&);
	Range(const double min, const double max);

	Range&				operator=(const Range&);
	bool				operator==(const Range&) const;
	bool				operator!=(const Range&) const;

	double				translateTo(double v, const Range&) const;

	double				clip(const double v) const;

	// Limit myself to the value.  Use this by setting min/max
	// to extremes (100000,-100000 or so) and then limiting each
	// value being processed, to find the final range.
	void				limitTo(const double v);

private:
	double				clipForward(const double v) const;
	double				clipBackward(const double v) const;
};

QDebug& operator<<(QDebug&, const Range&);

/*----------------------------------------------------------------*/
/*----- inline definitions ---------------------------------------*/
inline double Range::clipForward(const double v) const
{
	return (v < mMin ? mMin : (v > mMax ? mMax : v));
}

inline double Range::clipBackward(const double v) const
{
	return (v < mMax ? mMax : (v > mMin ? mMin : v));
}


#endif // RANGE_H
