#include "segment3.h"

#include <QPointF>
#include "math/geo.h"

/* SEGMENT-3
 ******************************************************************/
Segment3::Segment3()
{
}

Segment3::Segment3(const QPointF& a, const QPointF& b)
{
	set(a, b);
}

void Segment3::set(const Segment3& o)
{
	for (int k=0; k<SIZE; k++) mPt[k].set(o.mPt[k]);
}

void Segment3::set(const Point3& a, const Point3& b)
{
	mPt[0] = a;
	mPt[1] = b;
}

void Segment3::set(const QPointF& a, const QPointF& b)
{
	mPt[0].set(a.x(), a.y(), 0);
	mPt[1].set(b.x(), b.y(), 0);
}

double Segment3::distanceTo(const Point3& pt, Point3* nearest) const
{
	Point3			v = mPt[1] - mPt[0],
					w = pt - mPt[0];

	const double		c1 = w.dot3(v);
	if (c1 <= 0) {
		if (nearest) *nearest = mPt[0];
		return pt.dist3(mPt[0]);
	}

	const double		c2 = v.dot3(v);
	if (c2 <= c1) {
		if (nearest) *nearest = mPt[1];
		return pt.dist3(mPt[1]);
	}

	const double		b = c1/c2;
	v.x = (v.x * b) + mPt[0].x;
	v.y = (v.y * b) + mPt[0].y;
	if (nearest) *nearest = v;
	return pt.dist3(v);
}

double Segment3::distanceToSquared(const Point3& pt, Point3* nearest) const
{
	Point3				v = mPt[1] - mPt[0],
						w = pt - mPt[0];

	const double		c1 = w.dot3(v);
	if (c1 <= 0) {
		if (nearest) *nearest = mPt[0];
		return pt.dist3Squared(mPt[0]);
	}

	const double		c2 = v.dot3(v);
	if (c2 <= c1) {
		if (nearest) *nearest = mPt[1];
		return pt.dist3Squared(mPt[1]);
	}

	const double		b = c1/c2;
	v.x = (v.x * b) + mPt[0].x;
	v.y = (v.y * b) + mPt[0].y;
	if (nearest) *nearest = v;
	return pt.dist3Squared(v);
}

void Segment3::reverse()
{
	Point3			pt(mPt[0].x, mPt[0].y, mPt[0].z);
	mPt[0].set(mPt[1]);
	mPt[1].set(pt);
}

void Segment3::getFrame(QRectF& f) const
{
	geo::getFrame(mPt, SIZE, f);
}

QDebug& operator<<(QDebug& d, const Segment3& q)
{
	d << q.mPt[0] << " " << q.mPt[1];
	return d;
}
