#include "lineline.h"

#include <math.h>
#include <QtDebug>
#include"math/geo.h"

using namespace math;

/* MATH::LINE-LINE
 ******************************************************************/
LineLine::LineLine()
	: mEps(0.00001)
{
}

double LineLine::on(const Segment3& L1, const Segment3& L2)
{
	return on(L1.mPt[0], L1.mPt[1], L2.mPt[0], L2.mPt[1]);
}

// From http://www.mathworks.com/matlabcentral/newsreader/view_thread/246420
double LineLine::on(	const Point3& a1, const Point3& a2,
						const Point3& b1, const Point3& b2)
{
	mB2B1 = b2 - b1;
	mA1B1 = a1 - b1;
	mA2A1 = a2 - a1;

	mU.asCross3(mB2B1, mA1B1);
	mV.asCross3(mA2A1, mB2B1);
	const double		nA = mU.dot3(mV);
	mU.asCross3(mA2A1, mA1B1);
	mV.asCross3(mA2A1, mB2B1);
	const double		nB = mU.dot3(mV);
	mU.asCross3(mA2A1, mB2B1);
	mV.asCross3(mA2A1, mB2B1);
	const double		t = mU.dot3(mV);

	mA2A1 *= (nA/t);
	mNearest1 = a1 + mA2A1;
	mB2B1 *= (nB/t);
	mNearest2 = b1 + mB2B1;

//	qDebug() << "\t1=" << mNearest1 << " 2=" << mNearest2;

	return mNearest1.dist3(mNearest2);
}

bool LineLine::on2(const Point3& a1, const Point3& a2,
					const Point3& b1, const Point3& b2)
{
	m13.asSub(a1, b1);
	m43.asSub(b2, b1);
	if (geo::abs(m43.x) < mEps && geo::abs(m43.y) < mEps && geo::abs(m43.z) < mEps)
		return false;
	m21.asSub(a2, a1);
	if (geo::abs(m21.x) < mEps && geo::abs(m21.y) < mEps && geo::abs(m21.z) < mEps)
		return false;

	const double		d1343 = m13.x * m43.x + m13.y * m43.y + m13.z * m43.z;
	const double		d4321 = m43.x * m21.x + m43.y * m21.y + m43.z * m21.z;
	const double		d1321 = m13.x * m21.x + m13.y * m21.y + m13.z * m21.z;
	const double		d4343 = m43.x * m43.x + m43.y * m43.y + m43.z * m43.z;
	const double		d2121 = m21.x * m21.x + m21.y * m21.y + m21.z * m21.z;

	const double		denom = d2121 * d4343 - d4321 * d4321;
	if (geo::abs(denom) < mEps)
		return false;
	const double		numer = d1343 * d4321 - d1321 * d4343;

	mMua = numer / denom;
	mMub = (d1343 + d4321 * (mMua)) / d4343;

	mNearestA.x = a1.x + mMua * m21.x;
	mNearestA.y = a1.y + mMua * m21.y;
	mNearestA.z = a1.z + mMua * m21.z;
	mNearestB.x = b1.x + mMub * m43.x;
	mNearestB.y = b1.y + mMub * m43.y;
	mNearestB.z = b1.z + mMub * m43.z;

   return true;
}

// The softsurfer way.  I like it but I'm not clear on how to
// extract the nearest points
/*
	Point3		mU, mV, mW, mDp;
double LineLine::on(	const Point3& a1, const Point3& a2,
					const Point3& b1, const Point3& b2)
{
	mU = a2 - a1;
	mV = b2 - b1;
	mW = a1 - b1;
	const double		a = mU.dot3(mU),		// always >= 0
					b = mU.dot3(mV),
					c = mV.dot3(mV),		// always >= 0
					d = mU.dot3(mW),
					e = mV.dot3(mW);
	const double		D = a*c - b*b;			// always >= 0
	double			sc, tc;

	// compute the line parameters of the two closest points
	if (D < geo::EPS) {         // the lines are almost parallel
		sc = 0.0f;
		tc = (b>c ? d/b : e/c);   // use the largest denominator
	} else {
		sc = (b*e - c*d) / D;
		tc = (a*e - b*d) / D;
	}

	// get the difference of the two closest points
//	Vector   dP = mW + (sc * mU) - (tc * mV);  // = L1(sc) - L2(tc)
	mU *= sc;
	mV *= tc;
	mDp = mW + mU - mV;

	return mDp.length();		// return the closest distance
}
*/
