#include "lineplane.h"

#include <math.h>
#include "math/geo.h"

using namespace math;

/* MATH::LINE-PLANE
 ******************************************************************/
LinePlane::LinePlane()
{
	// This is what I'd like, but until I verify everything works...
//	mEps = geo::EPS;
	mEps = 0.000001;
}

int LinePlane::on(const Segment3& S, const Plane& Pn, Point3* outIp)
{
	return on(S.mPt[0], S.mPt[1], Pn, outIp);
}

int LinePlane::on(const Point3& a0, const Point3& a1, const Plane& Pn, Point3* outIp)
{
	mU = a1 - a0;
	mW = a0 - Pn.mPt;

	const double		D = Pn.mN.dot3(mU),
						N = -Pn.mN.dot3(mW);

	if (geo::abs(D) < mEps) {			// segment is parallel to plane
		if (N == 0) return INSIDE;		// segment lies in plane
		else return DISJOINT;			// no intersection
	}

	// they are not parallel
	// compute intersect param
	const double	sI = N / D;
	// If I want to bound to line segment?
//	if (sI < 0 || sI > 1) return DISJOINT;	// no intersection

	// compute segment intersect point
	if (outIp != NULL) {
		mU *= sI;
		*outIp = a0 + mU;
//		*I = S.P0 + sI * u;
	}
	return INTERSECT;
}
