#ifndef LINEPLANE_H
#define LINEPLANE_H

#include "math/primitives/plane.h"
#include "math/primitives/segment3.h"

namespace math {

/* MATH::LINE-PLANE
 * Intersection between an unbounded line and a plane
 * http://softsurfer.com/Archive/algorithm_0104/algorithm_0104B.htm
 ******************************************************************/
class LinePlane
{
public:
	static const int		INTERSECT = 1;		// Success, outIp is valid
	static const int		INSIDE = 0;			// Success? Depends on the context: segment lies inside plane
	static const int		ERROR = -1;
	static const int		DISJOINT = ERROR;

public:
	// Precision
	double					mEps;

public:
    LinePlane();

	// Segment L is actually an unbounded line, not a segment
	// Answer the intersection point at outIp.
	int						on(const Segment3&, const Plane&, Point3* outIp = NULL);
	int						on(const Point3& a0, const Point3& a1, const Plane& Pn, Point3* outIp = NULL);

private:
	Point3					mU, mW;
};

} // namespace math

#endif // LINEPLANE_H
