#include "polyisexternal.h"

#include <math.h>

using namespace math;

/* MATH::POLY-IS-EXTERNAL
 ******************************************************************/
PolyIsExternal::PolyIsExternal()
	: mEps(0.01)
	, mPrint(false)
{
	mLC.mEps = mEps;
	mSS.mEps = mEps;
	// Will likely never have to grow.
	mColinear.get(32);
}

bool PolyIsExternal::on(const Polygon& a, const Polygon& b)
{
if (mPrint) qDebug() << "PolyIsExternal";
	if (a.getSize() < 2 || b.getSize() < 2) return true;
/*
qDebug() << "POLY IS EXTERNAL";
qDebug() << "\ta " << a;
qDebug() << "\tb " << b;
qDebug() << "external(a,b)=" << isExternalTo(a, b) << " external(b, a)=" << isExternalTo(b, a);
*/

	// Weed out a specific, easy, fast case -- b's barycenter is inside a.
	a.getBarycenter(mPt);
	if (b.contains(mPt)) return false;
	b.getBarycenter(mPt);
	if (a.contains(mPt)) return false;

	return isExternalTo(a, b);
}

bool PolyIsExternal::isExternalTo(const Polygon& a, const Polygon& b)
{
	const qint32			sizeA = a.getSize();
	const Point3*			ptsA = a.getPoints();
	const qint32			sizeB = b.getSize();
	const Point3*			ptsB = b.getPoints();

	const Point3*			prvB = &(ptsB[sizeB-1]);

	// Reserve as much space as we could need for the colinears.
	Colinear*				col = mColinear.get((sizeA >= sizeB ? sizeA : sizeB) + 2);
	if (col == NULL) return true;
	int						colIdx = 0;

	for (qint32 kb=0; kb<sizeB; kb++) {
		const Point3&		nxtB = ptsB[kb];
if (mPrint) qDebug() << "b seg " << *prvB << "," << nxtB;
		// Compare this b segment against all of the segments in a.
		const Point3*		prvA = &(ptsA[sizeA-1]);
		int					idx = 0;
		for (qint32 ka=0; ka<sizeA; ka++) {
			const Point3&	nxtA = ptsA[ka];
			if (mLC.on(*prvA, nxtA, *prvB, nxtB)) {
if (mPrint) qDebug() << "\ta colinear seg " << *prvA << "," << nxtA;
				col[colIdx].mA0 = prvA;
				col[colIdx].mA1 = &nxtA;
				colIdx++;
			} else if (mSS.on2(*prvA, nxtA, *prvB, nxtB)) {
if (mPrint) qDebug() << "\ta seg " << *prvA << "," << nxtA;
				// The segments interesect.  Check to see if either endpoint
				// on b is inside poly a.  For performance we can toss a few
				// common pt == cases.
				if (isInternal(*prvB, a, *prvA, nxtA, mSS.mHit)) return false;
				if (isInternal(nxtB, a, *prvA, nxtA, mSS.mHit)) return false;

				// Otherwise look for multiple intersections against poly a.
				// If I hit two, I've possibly identified a fail condition,
				// although there are caveats:  At least one hit point can't
				// be on any colinear lines.
				mHit[idx].set(mSS.mHit);
				if (idx == 1) {
					if (looseHit(col, colIdx)) return false;
				} else idx = 1;
			}
			prvA = &nxtA;
		}

		prvB = &nxtB;
	}
	return true;
}

bool PolyIsExternal::looseHit(const Colinear* col, const int size) const
{
	if (size < 1) return true;
	for (int k=0; k<2; k++) {
		bool			loose = true;
		for (int kcol=0; kcol<size; kcol++) {
			if (col[kcol].sameAs(mHit[k], mEps)) {
				loose = false;
				break;
			}
		}
		if (loose) return true;
	}
	return false;
}

bool PolyIsExternal::isInternal(const Point3& pt, const Polygon& b)
{
	// If the point is inside the 3d polygon...
	if (mPiP.on(pt, b)) {
		// ...then how close is it to the edge?
		const double	d = mPtP.on(pt, b);
		if (d > mEps) {
			return true;
		}
	}
	return false;
}

bool PolyIsExternal::isInternal(	const Point3& pt, const Polygon& b,
									const Point3& edgeA, const Point3& edgeB, const Point3& edgeC)
{
	if (pt.sameAs3(edgeA, mEps)) return false;
	if (pt.sameAs3(edgeB, mEps)) return false;
	if (pt.sameAs3(edgeC, mEps)) return false;
	return isInternal(pt, b);
}

/* MATH::POLY-IS-EXTERNAL::COLINEAR
 ******************************************************************/
bool PolyIsExternal::Colinear::sameAs(const Point3& pt, const double eps) const
{
	return pt.sameAs3(*mA0, eps) || pt.sameAs3(*mA1, eps);
}
