#include "ptseg.h"

#include <math.h>

using namespace math;

/* MATH::PT-SEG
 ******************************************************************/
PtSeg::PtSeg()
{
}

double PtSeg::on(const Point3& pt, const Point3& a0, const Point3& a1)
{
	return sqrt(onSquared(pt, a0, a1, mHit));
}

double PtSeg::on(	const Point3& pt, const Point3& a0, const Point3& a1,
					Point3& out)
{
	return sqrt(onSquared(pt, a0, a1, out));
}

double PtSeg::onSquared(	const Point3& pt, const Point3& a0, const Point3& a1)
{
	return onSquared(pt, a0, a1, mHit);
}

double PtSeg::onSquared(	const Point3& pt, const Point3& a0, const Point3& a1,
							Point3& out)
{
	mV.asSub(a1, a0);
	mW.asSub(pt, a0);
	const double			c1 = mW.dot3(mV);
	if (c1 <= 0) {
		out.set(a0);
		return pt.dist3Squared(a0);
	}

	const double			c2 = mV.dot3(mV);
	if (c2 <= c1) {
		out.set(a1);
		return pt.dist3Squared(a1);
	}

	const double			b = c1 / c2;
	mPb.set(a0);
	mV *= b;
	mPb += mV;
	out.set(mPb);
	return pt.dist3Squared(mPb);
}
