#include "segintersection.h"

using namespace math;

/* MATH::SEG-INTERSECTION
 ******************************************************************/
SegIntersection::SegIntersection()
{
	mEps = 0.0001;
}

int SegIntersection::on(	const Point3& a0, const Point3& a1,
							const Point3& b0, const Point3& b1,
							Point3& out0, Point3& out1)
{
	// Shortcuts for identical segments
	if ( (a0.sameAs3(b0, mEps) && a1.sameAs3(b1, mEps))
			|| (a0.sameAs3(b1, mEps) && a1.sameAs3(b0, mEps)) ) {
		out0.set(a0);
		out1.set(a1);
		return INTERSECTS;
	}

	// Find the intersecting segment by finding the closest hits
	// (within tolerance) from each point to the opposite segment.
	mD[0] = mPS.on(a0, b0, b1, mHit[0]);
	mD[1] = mPS.on(a1, b0, b1, mHit[1]);
	mD[2] = mPS.on(b0, a0, a1, mHit[2]);
	mD[3] = mPS.on(b1, a0, a1, mHit[3]);

	// Collect the 2 nearest hits
	mNearest[0] = 0;
	mNearest[1] = 1;
	addNearest(2);
	addNearest(3);

	// Make sure both hits are within tolerance
	if (mD[mNearest[0]] > mEps || mD[mNearest[1]] > mEps) return ERROR;

	// Create a new segment from the hit points.
	out0 = mHit[mNearest[0]];
	out1 = mHit[mNearest[1]];

	return INTERSECTS;
}

void SegIntersection::addNearest(const int idx)
{
	// If the distance at the given index is less then one of my
	// current nearest, replace it.  If it's less then both, replace
	// the highest.
	const double			d = mD[idx];
	if (d < mD[mNearest[0]] && d < mD[mNearest[1]]) {
		if (mD[mNearest[0]] > mD[mNearest[1]]) mNearest[0] = idx;
		else mNearest[1] = idx;
	} else if (d < mD[mNearest[0]]) {
		mNearest[0] = idx;
	} else if (d < mD[mNearest[1]]) {
		mNearest[1] = idx;
	}
}
