#include "segplane.h"

#include <math.h>
#include "math/geo.h"

using namespace math;

/* MATH::SEG-PLANE
 ******************************************************************/
SegPlane::SegPlane()
{
	// This is what I'd like, but until I verify everything works...
//	mEps = geo::EPS;
	mEps = 0.000001f;
}


int SegPlane::on(const Point3& a1, const Point3& a2, const Plane& Pn)
{
	return on(a1, a2, Pn, mHit);
}

int SegPlane::on(const Point3& a1, const Point3& a2, const Plane& Pn, Point3& hit)
{
	mU = a2 - a1;
	mW = a1 - Pn.mPt;

	const double		D = Pn.mN.dot3(mU),
						N = -Pn.mN.dot3(mW);

	if (geo::abs(D) < mEps) {			// segment is parallel to plane
		if (N == 0) return INSIDE;		// segment lies in plane
		else return DISJOINT;			// no intersection
	}

	// they are not parallel
	// compute intersect param
	const double	sI = N / D;
	if (sI < 0 || sI > 1) return DISJOINT;	// no intersection

	// compute segment intersect point
	mU *= sI;
	hit.asAdd(a1, mU);

	return INTERSECT;
}
