#ifndef SEGPLANE_H
#define SEGPLANE_H

#include "math/primitives/plane.h"

namespace math {

/* MATH::LINE-PLANE
 * Intersection between an unbounded line and a plane
 * http://softsurfer.com/Archive/algorithm_0104/algorithm_0104B.htm
 ******************************************************************/
class SegPlane
{
public:
	static const int		INTERSECT = 1;		// Success, outIp is valid
	static const int		INSIDE = 0;			// Success? Depends on the context: segment lies inside plane
	static const int		ERROR = -1;
	static const int		DISJOINT = ERROR;

public:
	// Precision
	double					mEps;
	// A tmp point for clients that don't have one.
	Point3					mHit;

public:
	SegPlane();

	// Answer the intersection point at outIp.
	int						on(const Point3& a1, const Point3& a2, const Plane&);
	int						on(const Point3& a1, const Point3& a2, const Plane&, Point3& hit);

private:
	Point3					mU, mW;
};

} // namespace math

#endif // SEGPLANE_H
