#include "segseg.h"

#include "math/geo.h"

#include <math.h>

using namespace math;

/* MATH::SEG-SEG
 ******************************************************************/
SegSeg::SegSeg()
{
	// Ugh, framework is using this but should be the standard
	mEps = 0.00001;
//	mEps = 0.0000000001;
//	mEps = geo::EPS;
}

bool SegSeg::on(	const Segment3& S1, const Segment3& S2,
					Point3* nearest)
{
	return on(S1.mPt[0], S1.mPt[1], S2.mPt[0], S2.mPt[1], nearest);
}

bool SegSeg::on(	const Point3& a1, const Point3& a2,
					const Point3& b1, const Point3& b2,
					Point3* nearest)
{
	// First see if the unbounded lines intersect.
	mDistanceToHit = mLL.on(a1, a2, b1, b2);
//qDebug() << "distance=" << mDistanceToHit << " eps=" << mEps;
	if (mDistanceToHit >= mEps) return false;
	// Then make sure the point is on both segments.  We
	// treat the point nearest to the first segment as
	// "canonical."
//qDebug() << "\tps a=" << mPS.on(mLL.mNearest1, a1, a2, NULL);
//qDebug() << "\tps double a=" << don(mLL.mNearest1, a1, a2);
	if (mPS.on(mLL.mNearest1, a1, a2, mPS.mHit) >= mEps) return false;
	if (nearest) *nearest = mLL.mNearest1;
//qDebug() << "\tps b=" << mPS.on(mLL.mNearest1, b1, b2, NULL);
	return mPS.on(mLL.mNearest1, b1, b2, mPS.mHit) < mEps;

// OK, this is no doubt faster, but finding the actual intersection
// point does not work...  hmmm.
// XXX actually this is probably fine -- issue I was hitting was
// something weird where two tris weren't acting the same as the
// quad they came from
/*
	mU = a1 - a0;
	mV = b1 - b0;
	mW = a0 - b0;
	const float		a = mU.dot3(mU),		// always >= 0
					b = mU.dot3(mV),
					c = mV.dot3(mV),		// always >= 0
					d = mU.dot3(mW),
					e = mV.dot3(mW);
	const float		D = a*c - b*b;			// always >= 0
	float			sc, sN, sD = D;			// sc = sN / sD, default sD = D >= 0
	float			tc, tN, tD = D;			// tc = tN / tD, default tD = D >= 0

	// compute the line parameters of the two closest points
	if (D < geo::EPS) {		// the lines are almost parallel
		sN = 0.0;			// force using point P0 on segment S1
		sD = 1.0;			// to prevent possible division by 0.0 later
		tN = e;
		tD = c;
	} else {                // get the closest points on the infinite lines
		sN = (b*e - c*d);
		tN = (a*e - b*d);
		if (sN < 0.0) {       // sc < 0 => the s=0 edge is visible
			sN = 0.0;
			tN = e;
			tD = c;
		} else if (sN > sD) {  // sc > 1 => the s=1 edge is visible
			sN = sD;
			tN = e + b;
			tD = c;
		}
	}

	if (tN < 0.0) {           // tc < 0 => the t=0 edge is visible
		tN = 0.0;
		// recompute sc for this edge
		if (-d < 0.0)
			sN = 0.0;
		else if (-d > a)
			sN = sD;
		else {
			sN = -d;
			sD = a;
		}
	} else if (tN > tD) {      // tc > 1 => the t=1 edge is visible
		tN = tD;
		// recompute sc for this edge
		if ((-d + b) < 0.0)
			sN = 0;
		else if ((-d + b) > a)
			sN = sD;
		else {
			sN = (-d + b);
			sD = a;
		}
	}
	// finally do the division to get sc and tc
	sc = (geo::abs(sN) < geo::EPS ? 0.0 : sN / sD);
	tc = (geo::abs(tN) < geo::EPS ? 0.0 : tN / tD);

	// nearest
	if (nearest != NULL) *nearest = a0 - mU;

	// get the difference of the two closest points
//	Vector   dP = w + (sc * u) - (tc * v);
	mU *= sc;
	mV *= tc;
	mY = mU - mV;
	mDp = mW + mY;
	return mDp.length();
*/
}

bool SegSeg::on2(	const Point3& a1, const Point3& a2,
					const Point3& b1, const Point3& b2)
{
	// First see if the unbounded lines intersect.
	if (!mLL.on2(a1, a2, b1, b2)) return false;
	// Then make sure the point is on both segments.  We
	// treat the point nearest to the first segment as
	// "canonical."
	if ((mDistanceToHit=mPS.on(mLL.mNearestA, a1, a2, mPS.mHit)) >= mEps) return false;
	mHit.set(mLL.mNearestA);
	return (mDistanceToHit=mPS.on(mLL.mNearestA, b1, b2, mPS.mHit)) < mEps;
}
