#include "exportlist.h"

#include <QDebug>
#include "math/geo.h"

static const QString		JSON_FILE("file");
static const QString		JSON_RESIZE("resize");
static const QString		JSON_WIDTH("w");
static const QString		JSON_HEIGHT("h");
static const QString		JSON_FLAGS("flags");

/* EXPORT-LIST::FACTORY
 ******************************************************************/
class ExportList::Factory : public ManagedFactory<ExportList::Step*>
{
public:
	virtual ExportList::Step*		newObject() const {
		ExportList::Step*			ans = new ExportList::Step();
		if (ans == NULL) throw std::exception();
		return ans;
	}

	virtual void		deleteObject(ExportList::Step* t) const {
		delete t;
	}

	virtual void		initializeObject(ExportList::Step*) const {
	}

	virtual void		setObject(ExportList::Step* dst, ExportList::Step* src) const {
		if (src != NULL) dst->set(*src);
	}
};
static ExportList::Factory	FACTORY;

/* EXPORT-LIST
 ******************************************************************/
ExportList::ExportList()
	: mE(FACTORY)
{
}

int ExportList::size() const
{
	return mE.size();
}

void ExportList::clear()
{
	mE.clear();
}

ExportList::Step* ExportList::at(const int idx)
{
	if (idx < 0) return NULL;
	else if (idx >= mE.size()) return NULL;
	return mE.at(idx);
}

ExportList::Step* ExportList::append()
{
	return mE.pushBack();
}

void ExportList::swap(ExportList& o)
{
	mE.swap(o.mE);
}

void ExportList::print() const
{
	qDebug() << "ExportList size=" << mE.size();

	ManagedListIterator<Step*>		it(mE);
	while (it.hasValue()) {
		Step*						e = *it;
		++it;
		qDebug() << "\tfile=" << e->mFilename << " w=" << e->mW << " h=" << e->mH;
	}
}

/* EXPORT-LIST::STEP
 ******************************************************************/
ExportList::Step::Step()
	: mResize(RESIZE_OFF)
	, mW(1)
	, mH(1)
	, mFlags(F_LOCK_RATIO)
{
}

void ExportList::Step::set(const ExportList::Step& o)
{
	if (this != &o) {
		mFilename = o.mFilename;
		mResize = o.mResize;
		mW = o.mW;
		mH = o.mH;
		mFlags = o.mFlags;
	}
}

/* EXPORT-LIST IO
 ******************************************************************/
void ExportList::writeJson(json::Writer& w) const
{
	w.beginArray();

	ManagedListIterator<Step*>	it(mE);
	while (it.hasValue()) {
		Step*					e = *it;
		++it;
		if (e != NULL) e->writeJsonObject(w);
	}

	w.endArray();
}

void ExportList::startObject(json::Stack& s, const QString&)
{
	Step*			e = mE.pushBack();
	if (e != NULL) s.push(e);
}

/* EXPORT-LIST::STEP IO
 ******************************************************************/
void ExportList::Step::writeJsonObject(json::Writer& w) const
{
	w.beginObject();

	w.key(JSON_FILE);
	w.stringValue(mFilename);

	w.key(JSON_RESIZE);
	w.intValue(mResize);

	w.key(JSON_WIDTH);
	w.floatValue(mW);

	w.key(JSON_HEIGHT);
	w.floatValue(mH);

	w.key(JSON_FLAGS);
	w.intValue(mFlags);

	w.endObject();
}

void ExportList::Step::jsonString(json::Stack&, const QString& k, const QString& v)
{
	if (k == JSON_FILE) mFilename = v;
}

void ExportList::Step::jsonFloat(json::Stack&, const QString& k, const float v)
{
	if (k == JSON_RESIZE) mResize = geo::iroundf(v);
	else if (k == JSON_WIDTH) mW = v;
	else if (k == JSON_HEIGHT) mH = v;
	else if (k == JSON_FLAGS) mFlags = geo::iroundf(v);
}
