#ifndef EXPORTLIST_H
#define EXPORTLIST_H

#include "collections/managedlist.h"
#include "io/json.h"
#include "io/json_reader.h"

/* EXPORT-LIST
 * Collection of exports.
 ******************************************************************/
class ExportList : public json::Reader
		, public json::Writer::Client
{
public:
	static const int		RESIZE_OFF = 0;
	static const int		RESIZE_PERCENT = 1;
	static const int		RESIZE_PIXEL = 2;

	static const int		F_LOCK_RATIO = (1<<0);

	/* STEP
	 * Storage for one group of export settings.
	 **************************************************************/
	class Step : public json::Reader
	{
	public:
		Step();

		QString				mFilename;

		int					mResize;
		float				mW, mH;

		int					mFlags;

		void				set(const Step&);

		// IO
		void				writeJsonObject(json::Writer&) const;
		virtual void		jsonString(json::Stack&, const QString&, const QString&);
		virtual void		jsonFloat(json::Stack&, const QString&, const float);
	};

/* EXPORT-LIST
 ******************************************************************/
public:
    ExportList();

	int						size() const;
	void					clear();
	Step*					at(const int index);
	Step*					append();

	void					swap(ExportList&);

	// IO
	virtual void			writeJson(json::Writer&) const;
	virtual void			startObject(json::Stack&, const QString&);

	void					print() const;

private:
	ManagedList<Step*>		mE;

// internal
public:
	class Factory;
};

#endif // EXPORTLIST_H
