#include "project.h"

#include "engine/fx.h"
#include "math/geo.h"

static const QString	PROJECT_SZ("project");
static const QString	WIDTH_SZ("width");
static const QString	HEIGHT_SZ("height");
static const QString	EXPORT_SZ("export");

/* PROJECT
 ******************************************************************/
Project::Project()
	: mTimeStamp(0)
	, mCanvasSize(256, 256)
{
}

Project::~Project()
{
}

void Project::swap(Project& o)
{
	int			ts = mTimeStamp; mTimeStamp = o.mTimeStamp; o.mTimeStamp = ts;
	mHead.swap(o.mHead);
}

bool Project::copyFrom(const Project& prj)
{
	mTimeStamp = prj.mTimeStamp;

	const Fx*		fx = prj.mHead.fx();
	if (fx == NULL) {
		mHead.setFx(NULL);
		return true;
	}
	mHead.setFx(fx->copy());
	return mHead.fx() != NULL;
}

void Project::writeJson(json::Writer& w) const
{
	w.beginObject();

	w.key(json::DOC_SZ);
	w.stringValue(PROJECT_SZ);

	w.key(WIDTH_SZ);
	w.intValue(mCanvasSize.width());
	w.key(HEIGHT_SZ);
	w.intValue(mCanvasSize.height());

	w.key(EXPORT_SZ);
	mExports.writeJson(w);

	const Fx*		fx = mHead.fx();
	if (fx != NULL) {
		w.key(fx::FX_SZ);
		fx->writeJson(w);
	}

	w.endObject();
}

void Project::startObject(json::Stack& s, const QString& key)
{
	if (key == fx::FX_SZ) s.push(new fx::Reader(*this));
}

void Project::startArray(json::Stack& s, const QString& key)
{
	if (key == EXPORT_SZ) s.push(&mExports);
}

void Project::jsonFloat(json::Stack&, const QString& k, const float v)
{
	if (k == WIDTH_SZ && v > 0) mCanvasSize = QSize(geo::iroundf(v), mCanvasSize.height());
	else if (k == HEIGHT_SZ && v > 0) mCanvasSize = QSize(mCanvasSize.width(), geo::iroundf(v));
}

void Project::parseFx(Fx* fx)
{
	mHead.setFx(fx);
}
