#ifndef PROJECT_H
#define PROJECT_H

#include "engine/fxhead.h"
#include "engine/fxio.h"
#include "io/json.h"
#include "model/exportlist.h"

/* PROJECT
 * Store the settings for one project. which is a timestamp
 * and a series of FX units.
 ******************************************************************/
class Project : public json::Reader
			  , public fx::Reader::Target
{
public:
	Project();
	~Project();

	int					mTimeStamp;
	FxHead				mHead;
	QSize				mCanvasSize;
	// Store one or more objects describing how to export the
	// project to a standard image format.  Ideally, this whole
	// app would have some notion of batch processing, which
	// could encapsulate this.
	ExportList			mExports;

	// Exchange all data
	void				swap(Project&);

	// Copy the source data
	bool				copyFrom(const Project&);

	// IO
	void				writeJson(json::Writer&) const;
	virtual void		startObject(json::Stack&, const QString&);
	virtual void		startArray(json::Stack&, const QString&);
	virtual void		jsonFloat(json::Stack&, const QString&, const float);
	void				parseFx(Fx*);
};

#endif // PROJECT_H
