#ifndef PARAM_H
#define PARAM_H

#include <QString>
#include "param/paramlist.h"
#include "param/paramurl.h"

/* PARAM
 * Info about a single user-editable parameter.
 ******************************************************************/
class Param
{
public:
	static const int			FLOAT_TYPE = 0;
	static const int			STRING_TYPE = 1;
	// All below are free for client-defined types.
	static const int			CLIENT_TYPE = 512;

public:
    Param();
	~Param();

	param::Url		mUrl;
	QString			mGroupName,
					mParamName;
	int				mType;
	// Value is one of the following
	float			mFV;
	int				mIV;
	QString			mSV;
	// Params can also serve as groups of other params
	param::List		mList;

// XXX Note that mList is not copied on the copy constructor or
// assignment; the target param always has an empty list.  The
// problem is that Param is used as part of the signals and slots,
// so it needs support for those operations, but it doesn't make
// sense to ever do that on the list.  So probably there should
// be a separate simple param object that gets communicated to the
// FxUi::Entry, and this one should have these functions disabled.
public:
	Param(const Param&);
	Param&			operator=(const Param&);
};

#endif // PARAM_H
