#ifndef PARAMDEFS_H
#define PARAMDEFS_H

#include <QString>
#include "param/paramurl.h"

namespace param {

/* FILTER
 * Interface for clients to control what params are added
 * to a list.
 ******************************************************************/
class Filter {
public:
	Filter()				{ }
	virtual ~Filter()		{ }

	// Ehh...  pretty cheesy
	virtual bool accept(const QString* group, const QString* param, const int type) const = 0;
	// More cheesiness.  Features need to be defined by the client.
	virtual bool supports(const int featureFlags) const = 0;
};

// Predefined filters
Filter*		newNameFilter(const QString* group, const QString* param, const int features);
Filter*		newTypeFilter(const int type, const int features);

/* GET-ARGS
 * Arguments for getting params
 ******************************************************************/
class GetArgs
{
public:
	GetArgs();
	~GetArgs();

	// Set to an error state on failure
	int					mError;

	// Clients can supply a path to the parent for some automatic
	// limiting.  If no path, then the root of the project is used.
	// NOTE:  This is currently only used for the client to communicate
	// to the framework.  The param-holding objects responsible for
	// answering params have no use for this info, and it might disappear.
	Url					mUrl;

	// I don't own the filter, and won't delete it
	const Filter*		mFilter;

	// Shortcut that checks if the filter exists and supports the feature.
	bool				supports(const int featureFlags) const;

private:
	GetArgs(const GetArgs&);
	void				operator=(const GetArgs&);
};

// Private declarations
class NameFilter;
class TypeFilter;

} // namespace param

#endif // PARAMDEFS_H
