#ifndef PARAMLIST_H
#define PARAMLIST_H

#include <QList>
#include "param/paramdefs.h"

class Param;

namespace param {

class ListIterator;

/* PARAM::LIST
 * Store a list of params, encapsulate behaviour for manipulating
 * the list.
 ******************************************************************/
class List
{
public:
	List();
	~List();

	bool			isEmpty() const;

	// MANIPULATING
	// Conveniences for adding params to lists
	void			addParam(	const QString* group, const QString& name,
								const int paramId,
								const float fV,
								GetArgs&);
	void			addParam(	const QString* group, const QString& name,
								const int paramId,
								const QString& strV,
								GetArgs&);
	void			addParam(	const int type,
								const QString* group, const QString& name,
								const int paramId,
								GetArgs&,
								const float* floatV = NULL,
								const int* intV = NULL,
								const QString* strV = NULL);
	// Add a param that's nothing more than a type and ID.  The takeList,
	// if any, will be emptied, all items being placed in the type param.
	// There are some special rules at play:  The exp is an encoded value
	// for whether the UI should be expanded or contracted; the bit is
	// the flag bit for the expansion; and the key is the current key, if any.
	void			addTypeParam(	const QString& name,
									const int paramId,
									const int type,
									GetArgs&,
									const float exp = 0,
									const int bit = 0,
									const QString* key = NULL,
									List* takeList = NULL);

	// Remove everyone from src and place them in me
	void			takeAll(List&);

	// Prepend the ID to each param URL
	void			prepend(const int id);
	void			prepend(const Url&);

private:
	friend class ListIterator;

	class Node {
	public:
		explicit Node(Param*);
		~Node();

		Node*		mNext;
		Param*		mP;

		Node*		tail();
	};
	Node*			mHead;

	Param*			add(	const QString* group,
							const QString& name,
							const int paramId,
							const int type,
							GetArgs&);

// Utility for enforcing all incoming params to have the supplied group.
public:
	class Grouper {
	public:
		Grouper(List&, const QString& groupName);
		~Grouper();

	private:
		List&			mList;
		const QString	mGroupName;
		Node*			mPrev;

		void			assign(Node*);
	};
};

/* PARAM::LIST-ITERATOR
 ******************************************************************/
class ListIterator
{
public:
	ListIterator();
	ListIterator(const List&);

	void			start(const List&);
	void			operator++();

	bool			hasValue() const;
	const Param*	operator*();

private:
	List::Node*		mN;
};

} // namespace param

#endif // PARAMLIST_H
