#include "exportdialog.h"

#include <QBoxLayout>
#include <QDebug>
#include <QFileDialog>
#include <QLineEdit>
#include <QGridLayout>
#include <QLabel>
#include <QPushButton>
#include "model/activeproject.h"

static const QString			NO_SCALING_SZ("No resizing");
static const QString			PERCENT_SZ("Percent");
static const QString			PIXELS_SZ("Pixels");

/* EXPORT-DIALOG
 ******************************************************************/
ExportDialog::ExportDialog(ActiveProject& prj, QWidget * parent)
	: QDialog(parent)
	, mPrj(prj)
	, mCur(0)
	, mCanvasSize(prj.getCanvasSize())
	, mPercentSize(1, 1)
	, mPixelSize(prj.getCanvasSize())
	, mFile(NULL)
	, mScale(NULL)
	, mW(NULL)
	, mH(NULL)
{
	setSizeGripEnabled(false);

	// Initialize the export info
	prj.getExports(mExports);
	ExportList::Step*		e = mExports.at(mCur);
	if (e != NULL) {
		if (e->mResize == ExportList::RESIZE_PERCENT) mPercentSize = QSize(e->mW, e->mH);
		else if (e->mResize == ExportList::RESIZE_PIXEL) mPixelSize = QSize(e->mW, e->mH);
	} else {
		if ((e = mExports.append()) != NULL) {
			e->mResize = ExportList::RESIZE_OFF;
			e->mW = mCanvasSize.width();
			e->mH = mCanvasSize.height();
		}
	}

	mScaleMenu.addAction(NO_SCALING_SZ);
	mScaleMenu.addAction(PERCENT_SZ);
	mScaleMenu.addAction(PIXELS_SZ);

	QBoxLayout*			box = new QBoxLayout(QBoxLayout::TopToBottom);
	QBoxLayout*			row;
	QGridLayout*		grid = new QGridLayout;
	QLabel*				label;
	QLineEdit*			edit;
	QPushButton*		push;

	// SELECTION / FILE

	if ((row = new QBoxLayout(QBoxLayout::LeftToRight)) != NULL) {
		if ((push = new QPushButton("1")) != NULL) {
//			QObject::connect(push, SIGNAL(clicked()), this, SLOT(accept()));
			row->addWidget(push);
		}
		if ((push = new QPushButton("File")) != NULL) {
			QObject::connect(push, SIGNAL(clicked()), this, SLOT(onRequestFilename()));
			row->addWidget(push);
			mFile = push;
		}
		box->addLayout(row);
	}

	// SIZE

	if ((push = new QPushButton("Scale")) != NULL) {
//			QObject::connect(push, SIGNAL(clicked()), this, SLOT(accept()));
		push->setMenu(&mScaleMenu);
		grid->addWidget(push, 0, 1);
		mScale = push;
	}

	if ((label = new QLabel("width")) != NULL) grid->addWidget(label, 1, 0);
	if ((edit = new QLineEdit("0")) != NULL) {
		mW = edit;
		grid->addWidget(edit, 1, 1);
	}
	if ((label = new QLabel("height")) != NULL) grid->addWidget(label, 2, 0);
	if ((edit = new QLineEdit("0")) != NULL) {
		mH = edit;
		grid->addWidget(edit, 2, 1);
	}

	box->addLayout(grid);

	if ((row = new QBoxLayout(QBoxLayout::LeftToRight)) != NULL) {
		if ((push = new QPushButton("Ok")) != NULL) {
			QObject::connect(push, SIGNAL(clicked()), this, SLOT(accept()));
			row->addWidget(push);
		}
		if ((push = new QPushButton("Cancel")) != NULL) {
			QObject::connect(push, SIGNAL(clicked()), this, SLOT(reject()));
			row->addWidget(push);
		}
		box->addLayout(row);
		box->setAlignment(row, Qt::AlignRight);
	}

	setLayout(box);

	pushModelToUi();
}

void ExportDialog::accept()
{
	// Ideally the dialog is running asynchronously and supplies
	// the flattened exports back to the caller, but for now,
	// a direct set.
	mPrj.setExports(mExports);
	QDialog::accept();
}

void ExportDialog::onRequestFilename()
{
	const QString	fn = QFileDialog::getSaveFileName(this, tr("Save Image"), tr("/home"), tr("Image Files (*.png, *.9.png)"));
	if (fn.length() > 0) {
		ExportList::Step*		e = mExports.at(mCur);
		if (e != NULL) {
			e->mFilename = fn;
			if (mFile != NULL) mFile->setText(fn);
		}
	}
}

static const QString resizeText(const int s) {
	if (s == ExportList::RESIZE_PERCENT) return PERCENT_SZ;
	if (s == ExportList::RESIZE_PIXEL) return PIXELS_SZ;
	return NO_SCALING_SZ;
}

static void setNumberEdit(QLineEdit* edit, const float v, const bool disabled) {
	if (edit == NULL) return;
	QString				str = QString("%1").arg(v);
	edit->setText(str);
	edit->setEnabled(!disabled);
}

void ExportDialog::pushModelToUi()
{
	const ExportList::Step*		e = mExports.at(mCur);
	const int					off = ExportList::RESIZE_OFF;
	const int					resize = (e != NULL ? e->mResize : off);
	if (mFile != NULL) {
		if (e != NULL) mFile->setText(e->mFilename);
		else mFile->setText("");
	}
	if (mScale != NULL) mScale->setText(resizeText(resize));

	setNumberEdit(mW, e != NULL ? e->mW : 0, resize == off);
	setNumberEdit(mH, e != NULL ? e->mH : 0, resize == off);
}
