#ifndef RENDERVIEW_H
#define RENDERVIEW_H

#include <QPen>
#include <QPixmap>
#include <QPointF>
#include <QSize>
#include <QWidget>

#include "engine/fx.h"
#include "ui/framecontrol.h"
#include "renderthread.h"
class ActiveProject;

/* RENDER-VIEW
 * The widget that displays the main work area.
 ******************************************************************/
class RenderView : public QWidget
{
	Q_OBJECT
public:
	explicit RenderView(ActiveProject&, const Look&, QWidget *parent = 0);

	QSize					minimumSizeHint() const;
	QSize					sizeHint() const;

	// Rebuild any controls the render view displays
	void					onSelectFx(const int id);

	// Save my image according to the export rules
	void					writeExports(ActiveProject&);

signals:

public slots:
	void					onProjectChanged(const int changedFlags);
	void					onParamChanged(const Param&);
	void					onParamBatchFinished();

protected:
	void					mousePressEvent(QMouseEvent*);
	void					mouseMoveEvent(QMouseEvent*);
	void					mouseReleaseEvent(QMouseEvent*);
	void					paintEvent(QPaintEvent *event);

private:
	typedef QWidget	inherited;

	ActiveProject&			mPrj;
	RenderThread			mThread;

	// Set to true if I need to completely install the FX
	bool					mEmpty;

	// The bitmap buffer that gets painted
	QPixmap					mPixmap;
	QSize					mDataSize;

	// Control layers
	FrameControl			mFrameCtrl;
	RawControl*				mMouseCtrl;

	bool					anyControlsEnabled() const;

	void					write(const ExportList::Step&, const QPixmap&) const;

private slots:
	 void					onRenderFinished();

 // internal classes
 public:
	 class ParamFilter;
};

#endif // RENDERVIEW_H
