#include "settingsplug.h"

#include <assert.h>

#include <QDebug>
#include <QFileDialog>
#include <QPainter>

#include "model/activeproject.h"
#include "ui/look.h"
#include "ui/navigation.h"
#include "ui/stripview.h"
#include "ui/ui_defs.h"

/* SETTINGS-PLUG
 ******************************************************************/
SettingsPlug::SettingsPlug(	ActiveProject& prj, Navigation& nav,
							QWidget* parent)
	: mPrj(prj)
	, mNav(nav)
	, mQParent(parent)
	, mNeedsCache(true)
{
}

SettingsPlug::~SettingsPlug()
{
	uncache();
}

void SettingsPlug::setParent(FxUi::Parent* p, const Look& look)
{
	assert(p != NULL);
	inherited::setParent(p, look);

	if (mNeedsCache) {
		cache(p->entryH(), look);
		mNeedsCache = false;
	}

	for (int i=0; i<mE.size(); i++) {
		if (mE[i] == NULL) continue;
		p->add(mE[i]);
	}
}

int SettingsPlug::onProjectChanged()
{
	// XXX Don't really have a notion of identity for the project right now
	return FxUi::UI_UNCHANGED;
}

void SettingsPlug::cache(const int entryH, const Look& look)
{
	FxE*	e;
	if ((e = new FxE(*this, LOAD_CMD)) != NULL) mE.append(e);
	if ((e = new FxE(*this, SAVE_CMD)) != NULL) mE.append(e);
	if ((e = new FxE(*this, EXPORT_CMD)) != NULL) mE.append(e);
	if ((e = new FxE(*this, SIZE_CMD)) != NULL) mE.append(e);

	// Create images
	for (int i=0; i<mE.size(); i++) {
		if (mE[i] != NULL) mE[i]->cache(entryH, look);
	}
}

void SettingsPlug::uncache()
{
	for (int i=0; i<mE.size(); i++) delete mE[i];
	mE.clear();
}

/* SETTINGS-PLUG::FX-E
 *******************************/
SettingsPlug::FxE::FxE(SettingsPlug& plug, const QString& label)
	: mPlug(plug)
	, mLabel(label)
{
}

QSize SettingsPlug::FxE::size() const
{
	return mImg.size();
}

void SettingsPlug::FxE::onPaint(paint_args& pa)
{
	pa.mPainter->drawPixmap(pa.mFrame.left(), pa.mFrame.top(), mImg);
}

void SettingsPlug::FxE::mousePressEvent(const mouse_args&)
{
}

void SettingsPlug::FxE::mouseMoveEvent(const mouse_args&)
{
}

void SettingsPlug::FxE::mouseReleaseEvent(const mouse_args& down)
{
	if (down.mEntry == NULL) return;
	if (mLabel == LOAD_CMD) {
		mPlug.mNav.loadProject();
	} else if (mLabel == SAVE_CMD) {
		mPlug.mNav.saveProject();
	} else {
		mPlug.mNav.command(mLabel);
	}
}

void SettingsPlug::FxE::cache(const int entryH, const Look& look)
{
	if (mLabel.size() < 1) return;
	int					w = entryH, h = entryH;
	if (w < 1 || h < 1) return;
	QImage*				img = new QImage(w, h, QImage::Format_RGB888);
	if (img == NULL) return;

	{
		QPainter		painter(img);
		// Clear the background
		painter.setPen(Qt::NoPen);
		painter.setBrush(look.mDashBgB); //.mAlphaB);
		painter.drawRect(0, 0, w, h);
		// Draw a param panel
		painter.setBrush(look.mParamBgB);
		painter.setRenderHint(QPainter::Antialiasing);
		painter.drawRoundedRect(0, 0, w, h, look.mRound, look.mRound);
		// Draw the label
		painter.setPen(look.mParamFgC);
		painter.drawText(4, 10, mLabel);
	}

	// Kinda weird -- where's the error check?
	mImg = QPixmap::fromImage(*img);
	delete img;
}
