#ifndef STRIPVIEW_H
#define STRIPVIEW_H

#include <QWidget>
#include <QList>
#include "engine/fxui.h"

class ActiveProject;
class ArcList;
class Look;

/* STRIP-VIEW.  A single horizontal strip of controls.
 * I am a container for fxui's.
 ******************************************************************/
class StripView : public QWidget
		, public FxUi::Parent
{
	Q_OBJECT

// Hack for the arc browsing
public:
	ActiveProject&		mPrj;
	const ArcList&		mArcs;

public:
	StripView(ActiveProject&, const ArcList& arcs, const Look&, QWidget *parent = 0);
	~StripView();

	QSize			minimumSizeHint() const;

	// I become the owner of the plug.
	void			setPlug(FxUi*);

	virtual int		entryH() const;
	virtual void	add(FxUi::Entry*);
	virtual void	addBrowser(fxui::BrowserModel*, const int flags, const bool expanded);

public slots:
	void			onProjectChanged(const int changedFlags);
	void			onParamChanged(const Param&);

protected:
	void			mousePressEvent(QMouseEvent*);
	void			mouseMoveEvent(QMouseEvent*);
	void			mouseReleaseEvent(QMouseEvent*);
	void			paintEvent(QPaintEvent*);

private:
	// forward declarations
	class Holder;

	void			paintEntries(const bool drawOnTop);

	void			layoutEntries();
	const Holder*	find(const int x, const int y) const;

/* HOLDER.  The concrete entry holder
 *******************************/
private:
	class Holder {
	public:
		Holder();
		Holder(FxUi::Entry*);
		Holder(const Holder&);

		QRect			mFrame;
		FxUi::Entry*	mEntry;

		void			operator=(const Holder&);
	};

private:
	const Look&			mLook;

	// METRICS
	int					mEntryH;
	int					mViewH;

	bool				mLayout;	// Set to true when I need a layout

	FxUi*				mPlug;
	QList<Holder>		mList;
	FxUi::mouse_args	mDown;
	FxUi::paint_args	mPaintArgs;
};

#endif // STRIPVIEW_H
